// ***********************************
// AU2011_mainwindow.c
// ***********************************

#include "AU2011_mainwindow.h"
#include "ui_AU2011_mainwindow.h"
#include "qserialdevice/abstractserial.h"

#ifdef Q_OS_WIN
#include <windows.h>        // Sleep()
#endif

char Table_format[10][6]  = {"%1.0f","%1.1f","%1.2f",
                            "%1.1f","%1.2f","%1.3f",    // one more digit
                            "%1.2f","%1.3f","%1.4f",    // one more digit again
                            "%1.5f"};                   // one more digit again

char Table_format_QD[5][6] = {"%0.3f","%1.3f","%1.2f","%1.1f","%1.0f"};

// Array for temperature calculation of a PT100 probe
// index 0 not used. T_RTD[i] is temperature in °C for RTD = 10Ω * i
// ex: RTD = 100Ω, so i = 10 and  T_RTD[10] = 0.000 (°C).
// A linear interpolation between two consecutive values to obtain a precision of 0.05 °C
double T_RTD[41] = {0.0, -219.415, -196.509, -173.118, -149.304, -125.122, -100.617, -75.827, -50.781, -25.501, 0.000,
				   25.686, 51.571, 77.660, 103.958, 130.469, 157.198, 184.152, 211.336, 238.756, 266.419,
				   294.330, 322.498, 350.928, 379.628, 408.635, 437.889, 467.445, 497.309, 527.489, 557.993,
				   588.831, 620.014, 651.554, 683.464, 715.758, 748.453, 781.564, 815.110, 849.109, 883.582};


//-----------------------------------------------------------------------------
// AU2011_MainWindow Constructor
//-----------------------------------------------------------------------------
AU2011_MainWindow::AU2011_MainWindow(QWidget *parent) :
    QMainWindow(parent),
	ui(new Ui::AU2011_MainWindow)
{
    theLocal = QLocale::QLocale(); // locale of System if translation file found, or "C". See main.c file
    LocalizedDecimalPoint = theLocal.decimalPoint().toAscii();
    settings = new QSettings( QSettings::IniFormat, QSettings::UserScope,
                                                "FRAUBRJ", "AU2011 " + QString( AU2011_VERSION ) );
    ui->setupUi(this);

    if (settings->contains("myWidget/geometry"))
    {
        restoreGeometry(settings->value("myWidget/geometry").toByteArray());
        restoreState(settings->value("myWidget/windowState").toByteArray());
    }
    else
        AU2011_MainWindow::adjustSize ();   // Adjusts the size of the widget to fit its contents.

	QString message = trUtf8("   LCR Meter AU2011  version ","   LCR Mètre AU2011  version ");
	message.append(AU2011_VERSION);
    setWindowTitle(message);

    createActions();
    createMenus();

    port = new AbstractSerial(this);
	portDeviceName = "";
    if (settings->contains("port_name"))
    {
        portDeviceName = settings->value("port_name").toString( );
		port->setDeviceName(portDeviceName);
    }
    else
		GetPortName();

    // read the other settings
	if (settings->contains("myWidget/ledCheckBoxState"))
		ui->ledCheckBox->setChecked(settings->value("myWidget/ledCheckBoxState").toBool());
    else
        ui->ledCheckBox->setChecked(true);                                                      // default value

	if (settings->contains("myWidget/ErrorCheckBoxState"))
		ui->ErrorCheckBox->setChecked(settings->value("myWidget/ErrorCheckBoxState").toBool());
    else
        ui->ErrorCheckBox->setChecked(true);                                                    // default value

	if (settings->contains("myWidget/InfoGainCheckBoxState"))
		ui->InfoGainCheckBox->setChecked(settings->value("myWidget/InfoGainCheckBoxState").toBool());
    else
        ui->InfoGainCheckBox->setChecked(false);                                                // default value

	if (settings->contains("myWidget/AccuracyCheckBoxState"))
		ui->AccuracyCheckBox->setChecked(settings->value("myWidget/AccuracyCheckBoxState").toBool());

	if (settings->contains("SortingWidget/ChaineSort"))
		 chaineSort = settings->value("SortingWidget/ChaineSort").toString();

	if (settings->contains("SortingWidget/SortValue"))
         SortValue = settings->value("SortingWidget/SortValue").toDouble();

	if (settings->contains("SortingWidget/SortKind"))
		 SortKind = settings->value("SortingWidget/SortKind").toInt();

	if (settings->contains("SortingWidget/SortSuffix"))
         SortSuffix = settings->value("SortingWidget/SortSuffix").toDouble();

	if (settings->contains("SortingWidget/SortTolerance"))
         SortToleranceValue = settings->value("SortingWidget/SortTolerance").toDouble();

	int Display_Index;
	if (settings->contains("myWidget/DisplayUIIndex"))
		 Display_Index = settings->value("myWidget/DisplayUIIndex").toInt();
	else
		Display_Index = DISPLAY_U_I;
	ui->DisplayUIComboBox->setCurrentIndex(Display_Index);
	SetUIDisplay(Display_Index);

    connect(ui->openPort, SIGNAL(clicked()), this, SLOT(OpenSerialPort()));
    connect(ui->runButton, SIGNAL(clicked()), this, SLOT(RunStopMeasure()));
    connect(ui->Freq_1_Button, SIGNAL(toggled(bool)), this, SLOT(setFrequency(bool)));
    connect(ui->Freq_2_Button, SIGNAL(toggled(bool)), this, SLOT(setFrequency(bool)));
    connect(ui->Freq_3_Button, SIGNAL(toggled(bool)), this, SLOT(setFrequency(bool)));
    connect(ui->autoButton, SIGNAL(toggled(bool)),this, SLOT(setAutoMode(bool)));
    connect(ui->serieButton, SIGNAL(toggled(bool)),this, SLOT(setAutoMode(bool)));
    connect(ui->paraButton, SIGNAL(toggled(bool)),this, SLOT(setAutoMode(bool)));
    connect(ui->radioButton50Hz, SIGNAL(toggled(bool)),this, SLOT(setLineFrequency(bool)));
    connect(ui->radioButton60Hz, SIGNAL(toggled(bool)),this, SLOT(setLineFrequency(bool)));
    connect(ui->AverageSpinBox, SIGNAL(valueChanged(int)),this, SLOT(setAverageValue(int)));
	connect(ui->DisplayUIComboBox, SIGNAL(currentIndexChanged(int)),this,SLOT(SetUIDisplay(int)));
    connect(ui->offsetOKButton, SIGNAL(clicked()),this, SLOT(closeOffsetWidget()));
    connect(ui->trimPushButton, SIGNAL(clicked()),this, SLOT(Trim()));
    connect(ui->resistorValidationButton, SIGNAL(clicked()), this, SLOT(validateResistorWidget()));
    connect(ui->resistorCancelButton, SIGNAL(clicked()), this, SLOT(closeResistorWidget()));
    connect(ui->radioButton100k,SIGNAL(clicked()), this, SLOT(changeSelectedResistor()));
    connect(ui->radioButton10k,SIGNAL(clicked()), this, SLOT(changeSelectedResistor()));
    connect(ui->radioButton1k,SIGNAL(clicked()), this, SLOT(changeSelectedResistor()));
    connect(ui->radioButton100R,SIGNAL(clicked()), this, SLOT(changeSelectedResistor()));
    connect(ui->radioButton11,SIGNAL(clicked()), this, SLOT(setPhaseAdjustMultiplier()));
    connect(ui->radioButton101,SIGNAL(clicked()), this, SLOT(setPhaseAdjustMultiplier()));
    connect(ui->radioButton1001,SIGNAL(clicked()), this, SLOT(setPhaseAdjustMultiplier()));
    connect(ui->radioButton10001,SIGNAL(clicked()), this, SLOT(setPhaseAdjustMultiplier()));
    connect(ui->phiPlusMinusCheckBox,SIGNAL(stateChanged(int)), this, SLOT(setPlusMinusPhaseAdjustMultiplier(int)));
    connect(ui->phiValidationButton, SIGNAL(clicked()), this, SLOT(validatePhiWidget()));
    connect(ui->phiCancelButton, SIGNAL(clicked()), this, SLOT(closePhiWidget()));
    connect(ui->phiDataPlusButton, SIGNAL(clicked()), this, SLOT(increasePhiDataValue()));
    connect(ui->phiDataMinusButton, SIGNAL(clicked()), this, SLOT(decreasePhiDataValue()));
	connect(ui->sortValidationButton, SIGNAL(clicked()), this, SLOT(validateParamSortingWidget()));
    connect(ui->sortCancelButton, SIGNAL(clicked()), this, SLOT(closeParamSortingWidget()));
	connect(ui->SortLoadPushButton, SIGNAL(clicked()), this, SLOT(loadComponantParameters()));
	connect(ui->SortValueSpinBox, SIGNAL(valueChanged(double)), this, SLOT(SortValueChanged(double)));
	connect(ui->SortPushButton,SIGNAL(clicked()),this,SLOT(processSort()));
    connect(ui->adjustAccessCheckBox,SIGNAL(stateChanged(int)), this, SLOT(SetMenuReglages(int)));

	connect(ui->PT100CheckBox,SIGNAL(stateChanged(int)), this, SLOT(PT100_Checked(int)));
	connect(ui->AccuracyCheckBox, SIGNAL(stateChanged(int)), this, SLOT(setMaxPrecision(int)));
    connect(ui->Index_V_SpinBox, SIGNAL(valueChanged(int)),this, SLOT(setIndex_V_Value(int)));
    connect(ui->Index_I_SpinBox, SIGNAL(valueChanged(int)),this, SLOT(setIndex_I_Value(int)));

    connect(port, SIGNAL(readyRead()), this, SLOT(slotRead()), Qt::DirectConnection);

    trimTimer = new QTimer(this);
    connect(trimTimer, SIGNAL(timeout()), this, SLOT(updateTrimView()));

    connectionTimer = new QTimer(this);
    connectionTimer->setInterval(5000);
    connect(connectionTimer, SIGNAL(timeout()), this, SLOT(SendConnectedRequest()));

    ui->SortResultLabel->hide();
	ui->PT100CheckBox->hide();
	ui->PT100CheckBox->setChecked(false);
	ui->EraseBank2CheckBox->setChecked(false);
	MaxPrecision = ui->AccuracyCheckBox->isChecked();

#ifdef DEBUG_MODE_1
    ui->adjustAccessCheckBox->setChecked(true); // more convenient!
#else
    ui->adjustAccessCheckBox->setChecked(false); // normal use
#endif
    ui->adjustAccessCheckBox->setEnabled(false);
    ui->openPort->setEnabled(true);
    ui->runButton->setEnabled(false);
    ui->trimPushButton->setEnabled(false);
    ui->openTrimLabel->hide();
    ui->shortTrimLabel->hide();
    ui->SortPushButton->hide();
	ui->SortPushButton->setEnabled(true);
	ui->SortLoadPushButton->setEnabled(false);
	if (ui->InfoGainCheckBox->isChecked())
		 ui->GainFreqFrame->show();
	else
		ui->GainFreqFrame->hide();

    ui->Freq_1_Button->setCheckable(true);
    ui->Freq_2_Button->setCheckable(true);
    ui->Freq_3_Button->setCheckable(true);
	ui->ledLabel->hide();
	ui->ErrorCheckBox->hide();
    ui->FrameIndex->hide();
    ui->SchematicLabel->setPixmap(QPixmap(":/AU2011_Projet/Resources/StartScreen" ));

    // Tooltips.
	ui->RangeValueLabel->setToolTip(trUtf8("Measure range, from 1 to 8","Gamme de mesure, de 1 à 8"));
	ui->DAC_U_ValueLabel->setToolTip(trUtf8("Index of voltage post-amplification","Index de la post-amplication en tension"));
	ui->DAC_I_ValueLabel->setToolTip(trUtf8("Index of current post-amplification","Index de la post-amplication en courant"));
	ui->FreqValueLabel->setToolTip(trUtf8("Measurement frequency","Fréquence de la mesure"));
	ui->PrimaryValueLabel->setToolTip(trUtf8("Dominant parameter value","Valeur du paramètre dominant"));
	ui->SecondaryValueLabel->setToolTip(trUtf8("Secondary parameter value","Valeur du paramètre secondaire"));

    // There are some problems when '\n' is in the text for translation with trUtf8().
    // -> as the string is made by successive steps.
	message = trUtf8("Quality Factor ","Facteur de Qualité ");
    message.append('\n');
	message.append(trUtf8("or Dissipation Factor","ou Facteur de Dissipation"));
	ui->QD_ValueLabel->setToolTip(message);

	ui->PhiValueLabel->setToolTip(trUtf8("Phase angle","Angle de phase"));
	ui->Z_ValueLabel->setToolTip(trUtf8("Impedance","Impédance"));
	ui->SchematicLabel->setToolTip(trUtf8("Equivalent circuit","Circuit équivalent"));
	ui->openTrimLabel->setToolTip(trUtf8("Open Trim needed","Faire Le Trim circuit-ouvert"));
	ui->shortTrimLabel->setToolTip(trUtf8("Short Trim needed","Faire Le Trim circuit-fermé"));

	if (ui->ErrorCheckBox->isChecked())
		message = trUtf8("Max error due to the difference between the gains:","Erreur max due à la différence entre les gains:");
	else
		message = trUtf8("Typical error due to the difference between the gains:","Erreur typique due à la différence entre les gains:");
    message.append('\n');
	message.append(trUtf8(" - for voltage measurement"," - pour la mesure de la tension"));
    message.append('\n');
	message.append(trUtf8(" - for current measurement"," - pour la mesure du courant"));
	ui->ErrorValueLabel->setToolTip(message);

    double value = 1.1;
	ui->radioButton11->setText(theLocal.toString(value, 'f', 1));
	value = 1.01;
	ui->radioButton101->setText(theLocal.toString(value, 'f', 2));
	value = 1.001;
	ui->radioButton1001->setText(theLocal.toString(value, 'f', 3));
	value = 1.0001;
	ui->radioButton10001->setText(theLocal.toString(value, 'f', 4));

	ui->shortTrimLabel->setStyleSheet("color: red");
	ui->openTrimLabel->setStyleSheet("color: red");
    ui->ledLabel->setStyleSheet("color: green");
	ui->openPort->setFocus();

    AutoModel   = 0;
    SP_mode     = 0;
    AutoParameter	= 0;
    savedChoice = 0;
    Omega = 0x3a9 ;             // 'Ω'
    Limite_DAC_Index_V = -1;
    Limite_DAC_Index_I = -1;
    initialization = true;
    onMeasurement = false;
    onOffsetAdjust = false;
    onResistorsAdjust = false;
    onTrimAdjust = false;
    onInputPhaseAdjust = false;
    onPGAPhaseAdjust = false;
    onFirmwareUpdate = false;
    allow_DAC8811_32 = false;
    improvedOffset = false;
    checkConnectionAllowed = false;

    barreEtat = statusBar();
    QFont theFont = barreEtat->font();
    theFont.setPixelSize(13);
    barreEtat->setFont(theFont);

    // If a port name, try to open the port
    if (!portDeviceName.isEmpty())
        OpenSerialPort();

} // AU2011_MainWindow Constructor


//-----------------------------------------------------------------------------
// AU2011_MainWindow Destructor
//-----------------------------------------------------------------------------
AU2011_MainWindow::~AU2011_MainWindow()
{
	delete settings;
    delete port;
	delete trimTimer;
    delete connectionTimer;
    delete ui;

    qDebug() << "closing application";
}


//-----------------------------------------------------------------------------
// AU2011_MainWindow closeEvent
//-----------------------------------------------------------------------------
void AU2011_MainWindow::closeEvent(QCloseEvent *event)
 {
    if (onMeasurement)	// stop measurements
    {
		qDebug() << "send HALT";
		port->write(HALT);
		setDelay(100);
    }

	if(port->isOpen())
	{
		qDebug() << "send S_FREQ";
        port->write(SAVE_FREQ);		// save the current frequency
		setDelay(100);
    }

	port->close();

#ifdef DEBUG_MODE
	if (!port->isOpen())
		qDebug() << "Serial device : " << port->deviceName() << " closed.";
#endif

    // close some widget
    if (ui->offsetWidget->isVisible())
        ui->offsetWidget->hide();
    if (ui->phaseAdjustWidget->isVisible())
        ui->phaseAdjustWidget->hide();
    if (ui->senseResistorsWidget->isVisible())
        ui->senseResistorsWidget->hide();
    if (ui->SortingWidget->isVisible())
        ui->SortingWidget->hide();
    // Save a few settings
	settings->setValue("myWidget/geometry", saveGeometry());
    settings->setValue("myWidget/windowState", saveState());
	settings->setValue("myWidget/DisplayUIIndex", ui->DisplayUIComboBox->currentIndex());
    settings->setValue("myWidget/ledCheckBoxState", ui->ledCheckBox->isChecked());
	settings->setValue("myWidget/ErrorCheckBoxState", ui->ErrorCheckBox->isChecked());
	settings->setValue("myWidget/InfoGainCheckBoxState", ui->InfoGainCheckBox->isChecked());
	settings->setValue("myWidget/AccuracyCheckBoxState", ui->AccuracyCheckBox->isChecked());

    qDebug() << "close event";
    event->accept();
 }


//-----------------------------------------------------------------------------
// SLOT CloseApplication
//-----------------------------------------------------------------------------
void AU2011_MainWindow::CloseApplication(void)
{
	close();
}


//-----------------------------------------------------------------------------
// createActions
//-----------------------------------------------------------------------------
void AU2011_MainWindow::createActions(void)
{
	setPortNameAction = new QAction(trUtf8("Select port...","Sélectionnez le port..."), this);
    connect(setPortNameAction, SIGNAL(triggered()), this, SLOT(GetPortName()));

    closeThePortAction = new QAction(trUtf8("Close the port","Fermer le port"), this);
    connect(closeThePortAction, SIGNAL(triggered()), this, SLOT(OpenSerialPort()));

    bufferOffsetAction = new QAction(trUtf8("1 - Buffer_offset adjustement...","1 - Réglage buffer_offset..."), this);
    connect(bufferOffsetAction, SIGNAL(triggered()), this, SLOT(SetBufferOffset()));

    inputOffsetActionV = new QAction(trUtf8("2 - Input_offset_U adjustement...","2 - Réglage input_offset_U..."), this);
    connect(inputOffsetActionV, SIGNAL(triggered()), this, SLOT(SetInputOffset_V()));

    sinusOffsetAction = new QAction(trUtf8("3 - Sinus_offset adjustement...","3 - Réglage sinus_offset..."), this);
    connect(sinusOffsetAction, SIGNAL(triggered()), this, SLOT(SetSinusOffset()));

    inputOffsetActionI = new QAction(trUtf8("4 - Input_offset_I adjustement...","4 - Réglage input_offset_I..."), this);
    connect(inputOffsetActionI, SIGNAL(triggered()), this, SLOT(SetInputOffset_I()));

	resistorAction = new QAction(trUtf8("Sense resistors calibration...","Calibration des résistances..."), this);
    connect(resistorAction, SIGNAL(triggered()), this, SLOT(SetResistors()));

	inputPhaseAction = new QAction(trUtf8("Input phase adjustment...","Réglage phase entrée..."), this);
    connect(inputPhaseAction, SIGNAL(triggered()), this, SLOT(SetInputPhase()));

	PGAPhaseAction = new QAction(trUtf8("PGA phase adjustment...","Réglage phase PGA..."), this);
    connect(PGAPhaseAction, SIGNAL(triggered()), this, SLOT(SetPGAPhase()));

	FirmwareUpdateAction = new QAction(trUtf8("Firmware update...","Mise à jour programme..."), this);
    connect(FirmwareUpdateAction, SIGNAL(triggered()), this, SLOT(setFirmwareUpdate()));

#if defined (Q_OS_MACX)
    prefAction = new QAction("&Preferences", this);	// WARNING: don't translate. It's automatic
#else
	prefAction = new QAction(trUtf8("&Preferences...","&Préférences..."), this);
#endif
	prefAction->setShortcuts(QKeySequence::Preferences);
	connect(prefAction, SIGNAL(triggered()), this, SLOT(preferenceAct()));

	ParamSortAction = new QAction(trUtf8("Sort parameters...","Paramètres de tri..."), this);
    connect(ParamSortAction, SIGNAL(triggered()), this, SLOT(setSortingParameters()));

	OkForSortAction = new QAction(trUtf8("Activate Sorting","Activer le Tri"), this);
	connect(OkForSortAction, SIGNAL(triggered()), this, SLOT(sortAllowed()));

	SortAction = new QAction(trUtf8("Sort","Trier"), this);

#if defined (Q_OS_MACX)
    exitAction = new QAction("&Quit", this);	// WARNING: don't translate. It's automatic
#else
	exitAction = new QAction(trUtf8("&Quit","&Quitter"), this);
#endif
	exitAction->setShortcuts(QKeySequence::Quit);
	connect(exitAction, SIGNAL(triggered()),  this, SLOT(CloseApplication()));

#if defined (Q_OS_MACX)
    aboutAction = new QAction("&About", this);	// WARNING: don't translate. It's automatic
#else
    aboutAction = new QAction(trUtf8("&About AU2011","&À propos de AU2011"), this);
#endif
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(aboutAct()));

    aboutQtAction = new QAction(trUtf8("About Qt","À propos de Qt"), this);
    connect(aboutQtAction, SIGNAL(triggered()), this, SLOT(aboutQtAct()));

    bufferOffsetAction->setEnabled(false);
    //inputOffsetAction->setEnabled(false);
    inputOffsetActionV->setEnabled(false);
    inputOffsetActionI->setEnabled(false);
    sinusOffsetAction->setEnabled(false);
    resistorAction->setEnabled(false);

    inputPhaseAction->setEnabled(true);
    PGAPhaseAction->setEnabled(true);

    FirmwareUpdateAction->setEnabled(false);
    OkForSortAction->setEnabled(false);
}


//-----------------------------------------------------------------------------
// createMenus
//-----------------------------------------------------------------------------
void AU2011_MainWindow::createMenus(void)
{
    portMenu = menuBar()->addMenu(trUtf8("Port"));
    portMenu->addAction(setPortNameAction);
    portMenu->addAction(closeThePortAction);
    portMenu->addSeparator();
    portMenu->addAction(prefAction);
    portMenu->addSeparator();
    portMenu->addAction(exitAction);

    helpMenu = menuBar()->addMenu(trUtf8("Help","Aide"));
    helpMenu->addAction(aboutAction);
    helpMenu->addAction(aboutQtAction);
    closeThePortAction->setEnabled(false);

}


//-----------------------------------------------------------------------------
// addMenus
//-----------------------------------------------------------------------------
void AU2011_MainWindow::addMenus(void)
{
    menuBar()->clear();

    portMenu = menuBar()->addMenu(trUtf8("Port"));
    portMenu->addAction(setPortNameAction);
    portMenu->addAction(closeThePortAction);
#ifndef Q_OS_MACX
    portMenu->addSeparator();
    portMenu->addAction(prefAction);
    portMenu->addSeparator();
    portMenu->addAction(exitAction);
#endif
    outilsMenu = menuBar()->addMenu(trUtf8("Tools","Outils"));
    reglagesMenu = outilsMenu->addMenu(trUtf8("Calibrations","Réglages"));
    outilsMenu->addSeparator();
    outilsMenu->addAction(FirmwareUpdateAction);

    if (improvedOffset)
    {
        reglagesMenu->addAction(bufferOffsetAction);
        reglagesMenu->addAction(inputOffsetActionV);
        reglagesMenu->addAction(sinusOffsetAction);
        reglagesMenu->addAction(inputOffsetActionI);
        reglagesMenu->addSeparator();
        reglagesMenu->addAction(resistorAction);
        reglagesMenu->addAction(inputPhaseAction);
        reglagesMenu->addAction(PGAPhaseAction);
    }
    else
    {
        bufferOffsetAction->setText(trUtf8("Buffer_offset adjustement...","Réglage buffer_offset..."));
        inputOffsetActionI->setText(trUtf8("Input_offset adjustement...","Réglage input_offset..."));
        sinusOffsetAction->setText(trUtf8("Sinus_offset adjustement...","Réglage sinus_offset..."));
        reglagesMenu->addAction(inputOffsetActionI);
        reglagesMenu->addAction(sinusOffsetAction);
        reglagesMenu->addAction(resistorAction);
        reglagesMenu->addSeparator();
        reglagesMenu->addAction(bufferOffsetAction);
        reglagesMenu->addAction(inputPhaseAction);
        reglagesMenu->addAction(PGAPhaseAction);
    }

    ui->adjustAccessCheckBox->setEnabled(true);
    reglagesMenu->setEnabled(false);

    trierMenu = menuBar()->addMenu(trUtf8("Sort","Trier"));
    trierMenu->addAction(ParamSortAction);
    trierMenu->addSeparator();
    trierMenu->addAction(OkForSortAction);
    OkForSortAction->setCheckable(true);
    OkForSortAction->setChecked(false);

#ifndef Q_OS_MACX
    helpMenu = menuBar()->addMenu(trUtf8("Help","Aide"));
    helpMenu->addAction(aboutAction);
    helpMenu->addAction(aboutQtAction);
#endif

}


//-----------------------------------------------------------------------------
// SLOT preferenceAct
//-----------------------------------------------------------------------------
void AU2011_MainWindow::preferenceAct(void)
{
    if (port->isOpen())
            ui->prefDockWidget->setEnabled(true);
    ui->prefDockWidget->show();
}


//-----------------------------------------------------------------------------
// SLOT aboutAct
//-----------------------------------------------------------------------------
void AU2011_MainWindow::aboutAct(void)
{
    this->setEnabled(false);   // deleting parent (this) not allowed during the execution of the dialog
    QString version = trUtf8 ("LCR Meter AU2011 - Version ","LCR Mètre AU2011 - Version  ");
    version.append(AU2011_VERSION );
    version.append("\n  - Firmware ");
    version.append(firmwareVersion);
    version.append("\n  - Boot loader Version");
    version.append(bootloaderVersion);
    version.append("\n\n  2011-2013 - Jean-Jacques Aubry");
    QMessageBox::about(this, trUtf8("About LCR Meter AU2011","À propos du LCR Mètre AU2011"), version);
    this->setEnabled(true);
}


//-----------------------------------------------------------------------------
// SLOT aboutQtAct
//-----------------------------------------------------------------------------
void AU2011_MainWindow::aboutQtAct(void)
{
    this->setEnabled(false);   // deleting parent...
    QMessageBox::aboutQt(this, "");
    this->setEnabled(true);
}


//-----------------------------------------------------------------------------
// SLOT GetPortName
//
// For Unix machines (MacOSX and Linux) drivers are on "/dev" directory
// -    On MacOSX, the port name is "tty.usbserial-xxxxxxx"
// -    On Linux (Ubuntu) the port name is "ttyUSBx"
// For Windows machines, Com ports are COM1...COM10...etc.
//-----------------------------------------------------------------------------
void AU2011_MainWindow::GetPortName(void)
{

#if defined(Q_OS_WIN)
	char num[20];
	QStringList serialDeviceList;

    for (UINT i=1; i<256; i++)
	{
		//Form the Raw device name
        sprintf(num, "\\\\.\\COM%d", i);

		// Convert to wchar_t array
		int wchars_num =  MultiByteToWideChar( CP_UTF8 , 0 , num , -1, NULL , 0 );
		wchar_t *wstr = new wchar_t[wchars_num];
		MultiByteToWideChar( CP_UTF8 , 0 , num , -1, wstr , wchars_num );

		//Try to open the port
		HANDLE hPort = ::CreateFile(wstr, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);
		delete[] wstr;
		if (hPort != INVALID_HANDLE_VALUE)
		{
			//The port was opened successfully
            sprintf(num, "COM%d", i);
			serialDeviceList.append(num);
			CloseHandle(hPort);
		}
	}
	int j = serialDeviceList.size();
	if (j == 0)
		serialDeviceList.append("COMx");	// minimum one entry to be edited
#else
	QDir dir("/dev");
	dir.setFilter(QDir::System);
    QStringList serialDeviceList = dir.entryList().filter("USB", Qt::CaseInsensitive) ;
    serialDeviceList = serialDeviceList.filter("TTY", Qt::CaseInsensitive) ;
	int j = serialDeviceList.size();
	if (j == 0)
		serialDeviceList.append("");	// minimum one entry to be edited
#endif

	bool ok;
    this->setEnabled(false);   // deleting parent...
	QString portName = QInputDialog::getItem(this, trUtf8("Serial port","Port série"),
													trUtf8("Select the serial port","Choisissez le port à utiliser"), serialDeviceList, 0, true, &ok);
	this->setEnabled(true);

	 if (ok && !portName.isEmpty())
	 {
		QString namePort;
#if defined(Q_OS_WIN)
        namePort  = portName;                   // COMxx
#else
        if (portName.contains(QDir::separator ()))				// name edited with full path
			namePort = portName;
		else
			namePort  = "/dev/" + portName;		// choise in the list (not full path)
#endif
		if (!port->isOpen())
		{
			portDeviceName = namePort;
			port->setDeviceName(portDeviceName);
		}
		settings->setValue("port_name", namePort);
	 }
}


//-----------------------------------------------------------------------------
// SLOT OpenSerialPort
//-----------------------------------------------------------------------------
//
// when port opened, program continu echanging messages between GUI and hardware
//
void AU2011_MainWindow::OpenSerialPort(void)
{   
    if (port->isOpen())
    {
        if (onMeasurement)
            RunStopMeasure();
        setDelay(100);
        port->close();
        connectionTimer->stop();
        ClearDataInWindow();
        ui->openPort->show();
        ui->openPort->setText(trUtf8("Open port","Ouvrir le port"));
        ui->openPort->setFocus();
        ui->runButton->setEnabled(false);
        setPortNameAction->setEnabled(true);
        closeThePortAction->setEnabled(false);
        return;
    }

	QString message = trUtf8("connected to ","connecté à ");

    if ( !port->open(AbstractSerial::ReadWrite | AbstractSerial::Unbuffered) )
    {
        qDebug() << "Serial device : " << port->deviceName() << " open fail.";
        ui->openPort->setEnabled(true);
        ui->runButton->setEnabled(false);
		QString message = trUtf8("No connection to port ","Impossible de se connecter au port ");
        message.append(portDeviceName + '\n');
		message.append(trUtf8(" - check connection"," - vérifiez la liaison"));
        message.append('\n');
		message.append(trUtf8(" - check port's name"," - vérifiez le nom du port"));

        this->setEnabled(false);   // deleting parent...
		QMessageBox::warning(this, "",message);
        this->setEnabled(true);
    }
	else
	{
            qDebug() << "Serial device : " << port->deviceName() << " opened.";

            if (!port->setBaudRate(AbstractSerial::BaudRate115200)) {
                qDebug() << "Set baud rate " <<  AbstractSerial::BaudRate115200 << " error.";
            };
            if (!port->setDataBits(AbstractSerial::DataBits8)) {
                qDebug() << "Set data bits " <<  AbstractSerial::DataBits8 << " error.";
            }
            if (!port->setParity(AbstractSerial::ParityNone)) {
                qDebug() << "Set parity " <<  AbstractSerial::ParityNone << " error.";
            }
            if (!port->setStopBits(AbstractSerial::StopBits1)) {
                qDebug() << "Set stop bits " <<  AbstractSerial::StopBits1 << " error.";
            }
            if (!port->setFlowControl(AbstractSerial::FlowControlOff)) {
                qDebug() << "Set flow " <<  AbstractSerial::FlowControlOff << " error.";
            }

#if defined (Q_OS_UNIX)
            timeOut = 10;
			// Method setTotalReadConstantTimeout() not supported in *.nix.
           if (port->openMode() & AbstractSerial::Unbuffered)
                port->setCharIntervalTimeout(timeOut * 1000);    // us
#elif defined (Q_OS_WIN)
            timeOut = 50;
            if (port->openMode() & AbstractSerial::Unbuffered)
                port->setTotalReadConstantTimeout(timeOut);     // ms
#endif
            qDebug() << "= Port parameters =";
            qDebug() << "Device name            : " << port->deviceName();
            qDebug() << "Baud rate              : " << port->baudRate();
            qDebug() << "Data bits              : " << port->dataBits();
            qDebug() << "Parity                 : " << port->parity();
            qDebug() << "Stop bits              : " << port->stopBits();
            qDebug() << "Flow                   : " << port->flowControl();
#if defined (Q_OS_WIN)
            qDebug() << "Total read timeout constant, msec : " << port->totalReadConstantTimeout();
#else
            qDebug() << "Char interval timeout, usec       : " << port->charIntervalTimeout();
#endif

            ui->openPort->hide();
			ui->runButton->setFocus();
            ui->FreqGroupBox->setEnabled(true);
            ui->prefDockWidget->setEnabled(true);

			setPortNameAction->setEnabled(false);
            bufferOffsetAction->setEnabled(true);
            //inputOffsetAction->setEnabled(true);
            inputOffsetActionV->setEnabled(false);
            inputOffsetActionI->setEnabled(true);
            sinusOffsetAction->setEnabled(true);
            resistorAction->setEnabled(true);
            FirmwareUpdateAction->setEnabled(true);
            closeThePortAction->setEnabled(true);
			OkForSortAction->setEnabled(settings->contains("SortingWidget/SortSuffixIndex"));

            message.append(portDeviceName);
            barreEtat->showMessage(message, 10000);   // ms

            onMeasurement = false;
            messageReceived = false;

            // hardware is awaiting for first message (synchronization action)
#ifdef NO_BOOTLOADER
			qDebug() << "send GO_OFFSET";
			port->write(GO_OFFSET);				// synchrinization with firmware
#else
			qDebug() << "send START_BOOTLOADER";
			port->write(START_BOOTLOADER);		// synchrinization with bootloader
#endif
			//setDelay(1);
            QTimer::singleShot (1000, this, SLOT(CheckCom()));	// 1 sec later, check if hardware answered
	}
}	// OpenSerialPort


//-----------------------------------------------------------------------------
// SLOT slotRead
//-----------------------------------------------------------------------------
//
// SIGNAL readyRead (connected to this SLOT) emited when valid data are in Rx buffer.
//
void AU2011_MainWindow::slotRead(void)
{
	qint64 bw = port->readLine(inputString, 127);
	bool ok;

    messageReceived = true;
    if (checkConnectionAllowed)
        connectionTimer->start();

    // remove ending newline char
	if (inputString[bw-1] == '\n')
	{
		inputString[bw-1] = '\0';
		bw--;
	}

    inputQString = inputString;                 // inputString is a 'C' string (array of char)
#ifdef DEBUG_MODE
	qDebug() << "--> received " << bw << " bytes   " << inputString;
#endif
    if (inputQString.isEmpty() || bw <= 1)
        return;

    if ((bw > 2) && onMeasurement && (!onTrimAdjust))		// bw = 2 for "OK" and 2 characters message
	{
		if (ParseInputString())
		{
			ok = (Ranges == oldRanges);
			oldRanges = Ranges;
			if (!ok)							// if Ranges changed, don't display parameters
				return;
			ProcessAllParameters();
			if (AutoParameter == REACTIVE_CIRCUIT)
				ProcessCircuitXs();
			else if (AutoParameter == RESISTIVE_CIRCUIT)
				ProcessCircuitRs();
			Display_Frequency();
			Display_Q_D();
			Display_PHI();
			Display_Z();
			Display_Ranges();
			Display_Accuracy();
			Display_Symboles();
			Display_VxRs();
			Display_IxXs();
		}
		return;
	}

    if ( (bw > 2) &&
         ( !onMeasurement || onTrimAdjust || onResistorsAdjust || onInputPhaseAdjust || onPGAPhaseAdjust || onFirmwareUpdate) )
	{
        //qDebug() << bw << " bytes   " << inputString;
		ReadRequest();
		if (onOffsetAdjust)
			Display_Offset();
		return;
	}

    if (initialization && !onMeasurement)
	{
		ReadInitData();
		return;
	}

    //qDebug() << "--> received " <<  inputString;
}


//-----------------------------------------------------------------------------
// ReadRequest
//-----------------------------------------------------------------------------
//
// Filter the received string
//
void AU2011_MainWindow::ReadRequest(void)
{
     QString message;

     if (inputQString.contains(GO_APP))
     {
         qDebug() << "--> received " << "GO_APP";
         if (onFirmwareUpdate)
         {
             ui->SchematicLabel->setPixmap(QPixmap(":/AU2011_Projet/Resources/StartScreen" ));
             this->setEnabled(false);   // deleting parent...
             QMessageBox::information(this, "", trUtf8("Success on loading firmware.","Firmware chargé avec succès."));
             this->setEnabled(true);
             onFirmwareUpdate = false;
             addMenus();
             QTimer::singleShot (2000, this, SLOT(reactivateMenus()));
         }
         else
             setDelay(500);				// delay > 300ms needed by hardware (PWRN check)
         qDebug() << "send GO_OFS";
         port->write(GO_OFFSET);		// start offset's test
     }
     else if (inputQString.contains(VERSION, Qt::CaseInsensitive))
     {
         qDebug() << "--> received " << inputQString ;
         message = trUtf8("   LCR Meter AU2011  version ","   LCR Mètre AU2011  version ");
         message.append(AU2011_VERSION);
         message.append("    -     Firmware ");
         firmwareVersion = inputQString;
         message.append(firmwareVersion);
         setWindowTitle(message);

         // check compatibility with firmware version
         int spacePos = inputQString.lastIndexOf(' ');
         stringValue = inputQString.right(spacePos - 2);    // keeps only the version
         stringValue.remove(3,10);                          // keeps only the 2 first digits x.y
         double nombre = stringValue.toFloat() + 0.001;     // small value added for a good ">=" test!

         if (nombre >= FW_VERS_FOR_DAC8811_32)
         {
             Exx_Coeff = E32_COEFF;
             GMax_DAC = GMAX_DAC_32;
             allow_DAC8811_32 = true;
         }
         else
         {
             Exx_Coeff = E16_COEFF;
             GMax_DAC = GMAX_DAC_16;
             allow_DAC8811_32 = false;
         }

         checkConnectionAllowed = (nombre >= FW_VERS_FOR_CHECKCONNECT);
         if (checkConnectionAllowed)
             connectionTimer->start();
         qDebug() << "allow_DAC8811_32 is " << allow_DAC8811_32;
         qDebug() << "checkConnectionAllowed is " << checkConnectionAllowed;
     }
     else if (inputQString.contains(OVERFLOW_MSG))
     {
         qDebug() << "--> received " << "Overflow";
         ui->ledLabel->setStyleSheet("color: red");
         ui->ledLabel->show();
     }
     else if (inputQString.contains(SET_BUFFER_OFS))
     {
        qDebug() << "--> received " << "SetBufferOffset";
        this->setEnabled(false);   // deleting parent...
        message = trUtf8("Buffer offset adjustement needed.","Il faut régler l'offset du buffer.");
        message.append('\n');
        message.append(trUtf8("You have to open the case.","Pour cela ouvrez le boîtier."));
        message.append('\n');
        message.append(trUtf8("To continue?","Continuer?"));
        onOffsetAdjust = true;
        int reponse =  QMessageBox::question(this, "",message, QMessageBox::Yes | QMessageBox::No);
        this->setEnabled(true);
        if (reponse != QMessageBox::Yes)
        {
            WarningBeforeClosing();
            return;
        }
        SetBufferOffset();
     }
     else if (inputQString.contains(SET_INPUT_OFS_V))
     {
         qDebug() << "--> received " << "SetInputOffset_V";
         this->setEnabled(false);   // deleting parent...
         onOffsetAdjust = true;
         int reponse =  QMessageBox::question(this, "", trUtf8("First Input offset adjustement needed.","Il faut régler le premier offset de l'entrée."),
                                              QMessageBox::Yes | QMessageBox::No);
         this->setEnabled(true);
         if (reponse != QMessageBox::Yes)
         {
             WarningBeforeClosing();
             return;
         }
         SetInputOffset_V();
     }
     else if (inputQString.contains(IMPROVED_OFFSET))
     {
         qDebug() << "--> received " << "Improved_Offset";
         inputOffsetActionV->setEnabled(true);
         improvedOffset = true;
     }
     else if (inputQString.contains(SET_INPUT_OFS_I))
     {
         qDebug() << "--> received " << "SetInputOffset_I";
         this->setEnabled(false);   // deleting parent...
         onOffsetAdjust = true;
         if (improvedOffset)
             message = trUtf8("Second Input offset adjustement needed.","Il faut régler le second offset de l'entrée.");
         else
             message = trUtf8("Input offset adjustement needed.","Il faut régler l'offset de l'entrée.");
         int reponse =  QMessageBox::question(this, "", message,
                                               QMessageBox::Yes | QMessageBox::No);
          this->setEnabled(true);
          if (reponse != QMessageBox::Yes)
          {
              WarningBeforeClosing();
              return;
          }
          SetInputOffset_I();
     }
    else if (inputQString.contains(SET_SINUS_OFS))
    {
        qDebug() << "--> received " << "SetSinusOffset";
        this->setEnabled(false);   // deleting parent...
        onOffsetAdjust = true;
        int reponse =  QMessageBox::question(this, "", trUtf8("Sinus-generator offset adjustement needed.","Il faut régler l'offset du générateur."),
                                              QMessageBox::Yes | QMessageBox::No);
        this->setEnabled(true);
        if (reponse != QMessageBox::Yes)
        {
            WarningBeforeClosing();
            return;
        }
        SetSinusOffset();
    }
    else if (inputQString.contains(BAD_OFFSET))
    {
        qDebug() << "--> received " << "BAD_OFFSET";
        connectionTimer->stop();
        message = trUtf8("Impossible to perform this offset adjustement.","Impossible d'effectuer ce réglage d'offset.");
        message.append("\n");
        message.append(trUtf8("Closing the application.", "Fermeture de l'application."));
        message.append("\n");
        message.append(trUtf8("You must also shutdown the device", "Vous devez aussi arrêter l'appareil."));
        QMessageBox::information(this,"", message);
        CloseApplication();

    }
    else if (inputQString.contains(TRIM_OK))
    {
        qDebug() << "--> received " << "TRIM_ok";
        message = '\n';
        message.append(trUtf8(" Process ended with success"," Opération effectué avec succès"));
        ui->SchematicLabel->setText(message);
        onTrimAdjust = false;
        if (trimOperation == TRIM_SHORT)
        {
            trimShortState = true;
            ui->shortTrimLabel->hide();
        }
        else if (trimOperation == TRIM_OPEN)
        {
            trimOpenState = true;
            ui->openTrimLabel->hide();
        }
        trimOperation = NO_TRIM_OPE;
        savedChoice = 0;                            // force picture refresh
        ui->runButton->setEnabled(true);
        ui->FreqGroupBox->setEnabled(true);
        ui->ModeGroupBox->setEnabled(true);
        QTimer::singleShot (1000, this, SLOT(RunStopMeasure()));
    }
    else if (inputQString.contains(BAD_TRIM))
    {
        qDebug() << "--> received " << "Bad_TRIM";
        message = '\n';
        message.append(trUtf8("       WARNING","       ATTENTION"));
        message = "\n\n";
        message.append(trUtf8("  process failed","  l'opération a échoué"));
        ui->SchematicLabel->setText(message);
        onTrimAdjust = false;
        if (trimOperation == TRIM_SHORT)
        {
            trimShortState = false;
            ui->shortTrimLabel->show();
        }
        else if (trimOperation == TRIM_OPEN)
        {
            trimOpenState = false;
            ui->openTrimLabel->show();
        }
        trimOperation = NO_TRIM_OPE;
        savedChoice = 0;                        // force picture refresh
        ui->runButton->setEnabled(true);
        ui->FreqGroupBox->setEnabled(true);
        ui->ModeGroupBox->setEnabled(true);
        QTimer::singleShot (2000, this, SLOT(RunStopMeasure()));
    }
    else if (inputQString.contains(INIT))
    {
        qDebug() << "--> received " << "INIT";
        qDebug() << "send GOLOOP";
        setDelay(20);
        port->write(GO_LOOP);    // enter event loop
    }
    else  if (onOffsetAdjust)
    {
        Value_to_Print = inputQString.toDouble();
    }
    else if (onResistorsAdjust )
    {
        initialRsenseValue = inputQString.toDouble();
        ui->resistorValueSpinBox->setValue(initialRsenseValue);
    }
    else if ((onInputPhaseAdjust || onPGAPhaseAdjust) && !onMeasurement)
    {
        PhiData = inputQString.toDouble();
        qDebug() << "--> received Phida = " << PhiData;
        ui->phiPlusMinusCheckBox->setChecked(PhiData < 0.0);
        Display_PhiData();
        qDebug() <<  "   initial phidata : " << PhiData;
    }
    else  if (inputQString.contains(UPDATE_READY))
     {
        qDebug() << "--> received " << "UPDATE_FIRMWARE_READY";
        if(ui->EraseBank2CheckBox->isChecked())
        {
            port->write(UPDATE2);
            qDebug() << "send UPDATE2";
        }
        else
        {
            port->write(UPDATE);
            qDebug() << "send UPDATE";
        }
        setDelay(100);
     }
    else if (inputQString.contains(MEMORY_FAIL) || inputQString.contains(NEED_UPDATE))
    {
        bool ok = false;

        onFirmwareUpdate = true;
        if (inputQString.contains(MEMORY_FAIL))
        {
            qDebug() << "--> received " << "MEMORY_FAIL";
            QMessageBox::critical(this, "",
                               trUtf8("Program memory checksum error. You <strong>NECCESARILY</strong> have to reload the program.",
                                      "Le programme interne de l'appareil est corrompu. Vous devez <strong>IMPÉRATIVEMENT</strong> recharger un programme."));
        }
        else
        {
            qDebug() << "--> received " << "NEED_UPDATE";
            QMessageBox::critical(this, "",
                               trUtf8("User want to update the firmware.",
                                      "L'utilisateur demande une mise à jour du programme interne."));
        }
        ok = GetFileNameHex();
        if (!ok)
        {
            QMessageBox::information(this,"",trUtf8("  Closing the application. You must also shutdown the device",
                                                    "  Fermeture de l'application. Vous devez aussi arrêter l'appareil."));
            CloseApplication();
        }

        port->write(UPDATE);
        qDebug() << "send UPDATE";
        setDelay(100);
    }
    else if (inputQString.contains(READY_RECEIVE))
    {
        qDebug() << "--> received " << "READY_RECEIVE";
        qDebug() << "send the .hex file";
        SendFichierHex();
    }
    else if ( (inputQString.at(0).toAscii() == 'F') && (inputQString.at(3).toAscii() == 'P') && (inputQString.length() == 5))
    {
        char freqTest = inputQString.at(1).toAscii();
        //qDebug() << freqTest;
        switch (freqTest)
        {
            case '0':
                if (lineCode == FREQ_50)
                    strcpy(Buffer, "100 Hz");
                else
                    strcpy(Buffer, "120 Hz");
                break;
            case '1':
                strcpy(Buffer, "1 kHz");
                     break;
            case '2':
                strcpy(Buffer, "10 kHz");
                break;
        }
        message = '\n';
        if (trimOperation == TRIM_SHORT)
        {
            message.append(trUtf8(" Process Short Circuit Trim..."," Trim Circuit Fermé en cours..."));
        }
        else
        {
            message.append(trUtf8(" Process Open Circuit Trim..."," Trim Circuit Ouvert en cours..."));
        }
        message.append("\n\n");
        message.append (trUtf8("      Frequency : ","      Fréquence : "));
        message.append(Buffer);
        ui->SchematicLabel->setText(message);
    }
    else if (inputQString.contains("Bootloader"))
    {
        qDebug() << "--> received " << inputQString;
        inputQString.remove("Bootloader");
        bootloaderVersion = inputQString;
    }
    else
       qDebug() << inputQString;
}




//-----------------------------------------------------------------------------
// WarningBeforeClosing
//-----------------------------------------------------------------------------
//
// Displays a message before closing the application.
//
void AU2011_MainWindow::WarningBeforeClosing(void)
{
    QString message;

    connectionTimer->stop();
    message = trUtf8("You must set this offset before you continue.","Vous devez régler cet offset avant de poursuivre.");
    message.append("\n");
    message.append(trUtf8("Closing the application.", "Fermeture de l'application."));
    message.append("\n");
    message.append(trUtf8("You must also shutdown the device", "Vous devez aussi arrêter l'appareil."));
    QMessageBox::information(this,"", message);
    CloseApplication();
}


//-----------------------------------------------------------------------------
// ParseInputString
//
//		*** WARNING ***		Don't modify order of tests!
//		Input string is:	"Rs Xs Freq Ranges Vpp Ipp ADC_Vpp ADC_Ipp X"
//			Vpp & Ipp are multiplied by 0x400 to improve accuracy.
//			X = 'C' if component is capacitive, 'Z' if not.
//-----------------------------------------------------------------------------
bool AU2011_MainWindow::ParseInputString(void)
{
	int spacePos;
	bool ok;

    //qDebug() << inputQString;

	//** search for first string : Rs
	spacePos = inputQString.indexOf(' ');
    stringValue = inputQString.left(spacePos);
    RsValue = stringValue.toDouble(&ok);
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search for second string : Xs
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
    XsValue = stringValue.toDouble(&ok);
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search for third string : Freq
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
	FreqValue = stringValue.toUInt(&ok);
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search for fourth string : Ranges
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
	Ranges = stringValue.toUInt(&ok);
    Range_value = (Ranges & 0xF000) >> 12;
    dac_U = (Ranges & 0x07C0) >> 6;
    dac_I = Ranges & 0x001F;
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search for fifth string : Vpp
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
    VppValue = stringValue.toDouble(&ok) / 0x400;	// go back to true value
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search for sixth string : Ipp
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
    IppValue = stringValue.toDouble(&ok) / 0x400;	// go back to true value
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search for seventh string : ADC_Vpp
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
    ADC_Vpp = stringValue.toDouble(&ok);
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search for eighth string : ADC_Ipp
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
    ADC_Ipp = stringValue.toDouble(&ok);
	if (!ok)
		return false;
	inputQString.remove(0,spacePos+1);
	//** search if ninth string is "C" or not
	spacePos = inputQString.indexOf(' ');
	stringValue = inputQString.left(spacePos);
	isCapacitive = stringValue.contains('C');

    return ok;
}


//-----------------------------------------------------------------------------
// ReadInitData
//-----------------------------------------------------------------------------
//
// receive and set initialization parameters. Two characters, one letter and a value.
//
void AU2011_MainWindow::ReadInitData(void)
{

    char firstChar;
    short secondChar;
	bool ok = false;
	bool OK = false;
    bool x;

    qDebug() << inputQString;
	firstChar = inputQString.at(0).toAscii();
    inputQString.remove(0, 1);
    secondChar = inputQString.toShort();
	switch (firstChar) {
        case 'V':
            Limite_DAC_Index_V = inputQString.toShort(&x, 16);    // 0..9, A..F
            if (Limite_DAC_Index_V != -1)
            {
                ui->Index_V_SpinBox->setValue(Limite_DAC_Index_V);
                if (ui->adjustAccessCheckBox->isChecked())
                    ui->FrameIndex->show();
                 else
                    ui->FrameIndex->hide();
            }
            else
                ui->FrameIndex->hide();
            break;

        case 'I':
            Limite_DAC_Index_I = inputQString.toShort(&x, 16);    // 0..9, A..F
            if (Limite_DAC_Index_I != -1)
            {
                ui->Index_I_SpinBox->setValue(Limite_DAC_Index_I);
                if (ui->adjustAccessCheckBox->isChecked())
                    ui->FrameIndex->show();
                else
                    ui->FrameIndex->hide();
            }
            else
                ui->FrameIndex->hide();
            break;

		case 'L':
            if (secondChar == 5)
				lineCode = FREQ_50;
			else
				lineCode = FREQ_60;
			OK = true;	// more process needed
			break;
		case 'F':
            initFreqCode = secondChar;
			ok = true;	// more process needed
			break;
		case 'S':
            trimShortState = secondChar;
			break;
		case 'O':
            trimOpenState = secondChar;
			break;
		case 'A':
            addMenus();
            averagingValue = secondChar;
			ui->AverageSpinBox->setValue(averagingValue);
            ui->runButton->setEnabled(true);
			ui->runButton->setFocus();
			break;
	}

	if (OK)
	{
		switch(lineCode)
		{
		 case FREQ_50:
			ui->radioButton60Hz->setChecked(false);
			ui->radioButton50Hz->setChecked(true);
			ui->Freq_1_Button->setText(" 100 Hz");
			break;
		case FREQ_60:
			ui->radioButton50Hz->setChecked(false);
			ui->radioButton60Hz->setChecked(true);
			ui->Freq_1_Button->setText(" 120 Hz");
			break;
		}
		OK = FALSE;

		ui->radioButton50Hz->setEnabled(true);
		ui->radioButton60Hz->setEnabled(true);
		setDelay(2);
	}

    if (ok)
    {
        switch (initFreqCode)
        {
            case 0:
                ui->Freq_3_Button->setChecked(false);
                ui->Freq_2_Button->setChecked(false);
                ui->Freq_1_Button->setChecked(true);
                if (lineCode == FREQ_50)
                    FreqValue = 100;
                else
                    FreqValue = 120;
                break;
            case 1:
                ui->Freq_3_Button->setChecked(false);
                ui->Freq_1_Button->setChecked(false);
                ui->Freq_2_Button->setChecked(true);
                FreqValue = 1000;
                break;
            case 2:
                ui->Freq_1_Button->setChecked(false);
                ui->Freq_2_Button->setChecked(false);
                ui->Freq_3_Button->setChecked(true);
                FreqValue = 10000;
                break;
        }
        ok = false;

        ui->Freq_1_Button->setEnabled(true);
        ui->Freq_2_Button->setEnabled(true);
        ui->Freq_3_Button->setEnabled(true);
    }

	if (!trimOpenState || !trimShortState)
	{
		viewTrimWarning = true;
		trimTimer->start(1000);
	}
	else
		viewTrimWarning = false;
}


//-----------------------------------------------------------------------------
// SLOT SendConnectedRequest
//-----------------------------------------------------------------------------
//
// This slot is connected to "connectionTimer". At timeout()
//      sends a message to test if hardware always connected.
//
void AU2011_MainWindow::SendConnectedRequest(void)
{
    if (checkConnectionAllowed && !onPGAPhaseAdjust && !onInputPhaseAdjust && !onOffsetAdjust && !onResistorsAdjust && !onTrimAdjust && !onFirmwareUpdate)
    {
        messageReceived = false;
        qDebug() << "send QUERY_CONNECTED";
        port->write(QUERY_CONNECTED);
        QTimer::singleShot (2000, this, SLOT(CheckCom()));	// 2 sec later, check if hardware answered
    }
}


//-----------------------------------------------------------------------------
// SLOT CheckCom
//-----------------------------------------------------------------------------
//
// check if hardware send a message after receiving a message.
// It's a test of serial com Rx & Tx
//
void AU2011_MainWindow::CheckCom(void)
{
    if (!messageReceived)
    {
		 if (port->isOpen())
			 port->close();
        if (checkConnectionAllowed)
            connectionTimer->stop();
		QString message = trUtf8("No answer from the device on port ","L'appareil ne répond pas sur le port ");
        message.append(portDeviceName + '\n');
		message.append(trUtf8(" Port closed.  "," Connexion fermée."));
        message.append('\n');
		message.append(trUtf8(" Try again."," Essayez de nouveau."));
        this->setEnabled(false);   // deleting parent...
		QMessageBox::warning(this, "",message);
		this->setEnabled(true);
        setPortNameAction->setEnabled(true);
        closeThePortAction->setEnabled(false);
        if (onMeasurement)
            RunStopMeasure();
		ui->openPort->show();
		ui->runButton->setEnabled(false);
		ui->openPort->setFocus();
	}
}


//-----------------------------------------------------------------------------
// SLOT reactivateMenus
//-----------------------------------------------------------------------------
//
// called after an update of the firmware to set the calibration menu as before the update.
//
void AU2011_MainWindow::reactivateMenus(void)
{
    qDebug() << "reactivate Menus";
    if (ui->adjustAccessCheckBox->isChecked())
        reglagesMenu->setEnabled(true);

}


//-----------------------------------------------------------------------------
// SLOT SetMenuReglages
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetMenuReglages(int state)
{
    if  (state == Qt::Checked)
    {
        qDebug() << "Enable Menu Calibration";
        reglagesMenu->setEnabled(true);
        if (Limite_DAC_Index_V != -1 || Limite_DAC_Index_I != -1)
            ui->FrameIndex->show();
        else
            ui->FrameIndex->hide();
    }
    else
    {
        qDebug() << "Disable Menu Calibration";
        reglagesMenu->setEnabled(false);
        ui->FrameIndex->hide();
    }
}


//-----------------------------------------------------------------------------
// SLOT RunStopMeasure
//-----------------------------------------------------------------------------
//
// It's like a toggle
//
void AU2011_MainWindow::RunStopMeasure(void)
{
	qint64 bw;

	if (onMeasurement)	// stop measurements
	{
        qDebug() << "send HALT";
		bw = port->write(HALT);       
		onMeasurement = false;
		onTrimAdjust = false;
		ui->runButton->setText(trUtf8("Start","Démarrer"));
        ui->runButton->setFocus();
		ui->trimPushButton->setEnabled(false);
	}
	else
	{
        qDebug() << "send RUN";
		bw = port->write(RUN);
		onMeasurement = true;
		ui->SortLoadPushButton->setEnabled(true);
		ui->runButton->setText(trUtf8("Stop","Arrêter"));
		ui->runButton->clearFocus();
		ui->trimPushButton->setEnabled(true);
	}
}


//-----------------------------------------------------------------------------
// SLOT PT100_Checked
//-----------------------------------------------------------------------------
void AU2011_MainWindow::PT100_Checked(int)
{
	if(ui->PT100CheckBox->isChecked())
		ui->SecondaryValueLabel->setToolTip(trUtf8("Probe temperature","Température sonde"));
	else
		ui->SecondaryValueLabel->setToolTip(trUtf8("Secondary parameter value","Valeur du paramètre secondaire"));
}


//-----------------------------------------------------------------------------
// SLOT switchLedOff
//-----------------------------------------------------------------------------
void AU2011_MainWindow::switchLedOff(void)
{
	ui->ledLabel->hide();
}


//-----------------------------------------------------------------------------
// SLOT setAutoMode
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setAutoMode(bool checked)
{
	if (checked)
	{
		if (ui->autoButton->isChecked())
			AutoModel = 0;
		else if (ui->serieButton->isChecked())
			AutoModel = 1;
		else if (ui->paraButton->isChecked())
			AutoModel = 2;
	}
}

//-----------------------------------------------------------------------------
// SLOT setMaxPrecision
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setMaxPrecision(int)
{
	MaxPrecision = ui->AccuracyCheckBox->isChecked();
}

//-----------------------------------------------------------------------------
// SLOT setLineFrequency
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setLineFrequency(bool checked)
{
	if (checked)
	{
		if (ui->radioButton50Hz->isChecked())
		{
			port->write(LINE5);
            qDebug() << "send LINE5";
			lineCode = FREQ_50;
			ui->Freq_1_Button->setText(" 100 Hz");
		}
		else if (ui->radioButton60Hz->isChecked())
		{
			port->write(LINE6);
            qDebug() << "send LINE6";
			lineCode = FREQ_60;
			ui->Freq_1_Button->setText(" 120 Hz");
		}
	}
}


//-----------------------------------------------------------------------------
// SLOT SetUIDisplay
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetUIDisplay(int index)
{
    switch(index)
    {
        case DISPLAY_R_X:
			ui->VxRsLabel->setText("Rs");
			ui->VxRsValueLabel->setToolTip(trUtf8("Equivalent series resistance","Résistance série équivalente"));
            ui->IxXsLabel->setText("Xs");
			ui->IxXsValueLabel->setToolTip(trUtf8("Equivalent series reactance","Réactance série équivalente"));
            break;
		case DISPLAY_U_I:
			ui->VxRsLabel->setText("Ux");
			ui->VxRsValueLabel->setToolTip(trUtf8("Componant test voltage","Tension du composant en test"));
            ui->IxXsLabel->setText("Ix");
			ui->IxXsValueLabel->setToolTip(trUtf8("Componant test current","Courant du composant en test"));
            break;
         case DISPLAY_ADC:
			ui->VxRsLabel->setText("ADC U");
			QString message = trUtf8("ADC input voltage","Tension en entrée ADC");
            message.append('\n');
			message.append(trUtf8("for Voltage measurement","pour la mesure Tension"));
			ui->VxRsValueLabel->setToolTip(message);
            ui->IxXsLabel->setText("ADC I");
			message = trUtf8("ADC input voltage","Tension en entrée ADC");
            message.append('\n');
			message.append(trUtf8("for Current measurement","pour la mesure Courant"));
			ui->IxXsValueLabel->setToolTip(message);
            break;
    }
}


//-----------------------------------------------------------------------------
//SLOT setFrequency
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setFrequency(bool checked)
{
	if (checked)
	{
        qDebug() << "send FREQ_X";
		if (ui->Freq_1_Button->isChecked())
			port->write(FREQ_1);
		else if (ui->Freq_2_Button->isChecked())
			port->write(FREQ_2);
		else if (ui->Freq_3_Button->isChecked())
			port->write(FREQ_3);
	}
}


//-----------------------------------------------------------------------------
// SLOT setAverageValue
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setAverageValue(int Value)
{
    char chaine[10];

	if (Value == averagingValue)
		return;
	// construct "AVG_x\n", x = 1 to 9
    sprintf(chaine, AVERAG_x, Value);
	port->write(chaine);
    qDebug() << "send " << chaine;
	averagingValue = Value;
	setDelay(200);
}


//-----------------------------------------------------------------------------
// SLOT setIndex_V_Value
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setIndex_V_Value(int Value)
{
    char chaine[15];

    if (Value == Limite_DAC_Index_V)
        return;
    // construct "INDXV x\n", x = 0 to F
    sprintf(chaine, "INDXV %X", Value);
    strcat(chaine,"\n");
    port->write(chaine);
    Limite_DAC_Index_V = Value;
    qDebug() << "send " << chaine;
    setDelay(200);
}


void AU2011_MainWindow::setIndex_I_Value(int Value)
{
    char chaine[15];

    if (Value == Limite_DAC_Index_I)
        return;
    // construct "INDXI x\n", x = 0 to F
    sprintf(chaine, "INDXI %X", Value);
    strcat(chaine,"\n");
    port->write(chaine);
    Limite_DAC_Index_I = Value;
    qDebug() << "send " << chaine;
    setDelay(200);
}


//-----------------------------------------------------------------------------
// GetFileNameHex
//-----------------------------------------------------------------------------
bool AU2011_MainWindow::GetFileNameHex(void)
{
    this->setEnabled(false);   // deleting parent...
	nomFichierHex = QFileDialog::getOpenFileName(this, trUtf8("Select the file","Choisir le fichier"), QString(), "Text files (*.hex)");
	if (nomFichierHex.isEmpty())
    {
        qDebug() << "cancel";
		this->setEnabled(true);
        return false;
    }
    return true;
}


//-----------------------------------------------------------------------------
// SendFichierHex
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SendFichierHex(void)
{
    char buf[128];
    qint64 lineLength, bytesLoaded = 0;
    QString message;
    QProgressDialog progress(this);

    QFile file(nomFichierHex);
    qint64 TotalSize = file.bytesAvailable();

    QString nomFichier = nomFichierHex;
    int slashPos = nomFichier.lastIndexOf('/');     // works for all OS
    nomFichier.remove(0, slashPos + 1);             // remove path
    message = trUtf8("loading ","chargement de ");
    message.append(nomFichier);
    progress.setLabelText(message);
    progress.setRange(0, 100);
    progress.setModal(true);
    progress.setCancelButton(0);

    if (file.open(QFile::ReadOnly))
    {
        while (!file.atEnd())
        {
           lineLength = file.readLine(buf, sizeof(buf));
           if (lineLength != -1)
           {
               port->write(buf);
               bytesLoaded += lineLength;
               progress.setValue(bytesLoaded * 100 / TotalSize);
               //setDelay(1);
           }
        }
        file.close();
    }
}


//-----------------------------------------------------------------------------
// SLOT setFirmwareUpdate
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setFirmwareUpdate(void)
{
    connectionTimer->stop();
    this->setEnabled(false);   // deleting parent...
	int reponse = QMessageBox::question(this, "", trUtf8("Do you really want to update the firmware of the device?",
														 "Voulez-vous vraiment effectuer la mise à jour du programme interne de l'appareil?"),
                          QMessageBox::Yes | QMessageBox::Cancel, QMessageBox::Cancel);

    if (reponse != QMessageBox::Yes)
	{
		this->setEnabled(true);
        return;
	}
	if (ui->EraseBank2CheckBox->isChecked())
	{
        reponse = QMessageBox::warning(this, "", trUtf8("The calibrations will be erased. Continue?",
                                                                 "Les réglages seront effacés. Continuer?"),
								  QMessageBox::Yes | QMessageBox::Cancel, QMessageBox::Cancel);
	}
	this->setEnabled(true);
	if (reponse != QMessageBox::Yes)
		return;

    // if running, stop measurements
    if (onMeasurement)
    {
        RunStopMeasure();
        setDelay(20);
	}

    if (GetFileNameHex())
    {
		setDelay(100);
        // reset the hardware
		onFirmwareUpdate = true;
        qDebug() << "send FWUPDA";
		port->write(FIRMW_UPDATE);
        // reset the windows title (without firmware version)
        QString title = trUtf8("   LCR Meter AU2011  version ","   LCR Mètre AU2011  version ");
        title.append(AU2011_VERSION);
        setWindowTitle(title);
        setDelay(20);
    }
	else
		this->setEnabled(true);
}



//-----------------------------------------------------------------------------
// updateTrimView
//-----------------------------------------------------------------------------
void AU2011_MainWindow::updateTrimView(void)
{
	if (viewTrimWarning)
	{
		if (!trimShortState)
			ui->shortTrimLabel->show();
		if (!trimOpenState)
			ui->openTrimLabel->show();
		viewTrimWarning = false;
	}
	else
	{
		if (!trimShortState)
			ui->shortTrimLabel->hide();
		if (!trimOpenState)
			ui->openTrimLabel->hide();
		viewTrimWarning = true;
	}
	if (trimOpenState && trimShortState)
		trimTimer->stop();
}




//-----------------------------------------------------------------------------
// ClearDataInWindow
//-----------------------------------------------------------------------------
void AU2011_MainWindow::ClearDataInWindow(void)
{
	ui->PrimaryValueLabel->setText("");
	ui->PrimaryUnityLabel->setText("");
	ui->SecondaryValueLabel->setText("");
	ui->SecondaryUnityLabel->setText("");
	ui->QD_ValueLabel->setText("");
	ui->label_QD->setText("");
	ui->PhiValueLabel->setText("");
	ui->PhiUnityLabel->setText("");
	ui->Z_ValueLabel->setText("");
	ui->Z_UnityLabel->setText("");
	ui->VxRsValueLabel->setText("");
	ui->VxRsUnityLabel->setText("");
	ui->IxXsValueLabel->setText("");
	ui->IxXsUnityLabel->setText("");
	ui->RangeValueLabel->setText("");
	ui->DAC_U_ValueLabel->setText("");
	ui->DAC_I_ValueLabel->setText("");
	ui->ErrorValueLabel->setText("");
	ui->SchematicLabel->setPixmap(QPixmap(":/AU2011_Projet/Resources/StartScreen"));
	savedChoice = 0;
}




// *************  SORTING ***************
//-----------------------------------------------------------------------------
// SLOT processSort
//-----------------------------------------------------------------------------
void AU2011_MainWindow::processSort(void)
{
    double maxValue = SortValue * (1 + SortToleranceValue);
    double minValue = SortValue * (1 -SortToleranceValue);
	QString chaine = trUtf8("Real deviation : ","Ecart réel : ");
	int p, timeToSee;

	ui->SortPushButton->setEnabled(false);

	bool ok = (SortKind == primaryKind || (SortKind == R_KIND && primaryKind == T_KIND));
	if (ok)
		ok = ( (PrimaryValue >= minValue) && (PrimaryValue <= maxValue) );

	double deviation = (PrimaryValue - SortValue) / SortValue * 100;

	if (fabs(deviation) > 99.99)
		chaine.append("> 100 %");
	else
	{

		if (fabs(deviation) < 10.0)
			p = 1;
		else
			p = 0;
		if (!ok)
			p++;
		QString tolerance = theLocal.toString(deviation, 'f', p);
		chaine.append(tolerance + " %");
	}

	if (ok)
	{
		ui->SortResultLabel->setStyleSheet("color: green");
		timeToSee = 2000;
	}
	else
	{
		ui->SortResultLabel->setStyleSheet("color: red");
		timeToSee = 1200;
	}

	ui->SortResultLabel->setText(chaine);
	ui->SortResultLabel->show();

	QTimer::singleShot (timeToSee, this, SLOT(hideSortResultLabel()));
}


//-----------------------------------------------------------------------------
// SLOT hideSortResultLabel
//-----------------------------------------------------------------------------
void AU2011_MainWindow::hideSortResultLabel(void)
{
	ui->SortPushButton->setEnabled(true);
	ui->SortResultLabel->hide();
	ui->SortPushButton->setFocus();
}


//-----------------------------------------------------------------------------
// SLOT loadComponantParameters
//-----------------------------------------------------------------------------
void AU2011_MainWindow::loadComponantParameters(void)
{
	 bool ok;
	 int suffix;
	 int index;

	 QString chaine = ui->PrimaryUnityLabel->text();

	 //qDebug() << "Primary label: " << chaine;
	 if (chaine.contains(Omega))
		  ui->SortKindComboBox->setCurrentIndex(R_KIND - 1);
	 else  if (chaine.contains('H'))
		  ui->SortKindComboBox->setCurrentIndex(L_KIND - 1);
	 else if (chaine.contains('F'))
		  ui->SortKindComboBox->setCurrentIndex(C_KIND - 1);

	 if ( chaine.length() == 2)
	 {
		  suffix = chaine.at(0).unicode();
		  switch (suffix)
		  {
				case 'p':
					 index = 0;
					 break;
				case 'n':
					 index = 1;
					 break;
				case 0x3bc:		// mu
					 index = 2;
					 break;
				case 'm':
					 index = 3;
					 break;
				case 'k':
					 index = 5;
					 break;
				case 'M':
					 index = 6;
					 break;
		  }
	 }
	 else
		  index = 4;
	 ui->SortSuffixComboBox->setCurrentIndex(index);

	 chaine = ui->PrimaryValueLabel->text();
     double value = chaine.toDouble(&ok);
	 qDebug() << "chaine: "  << chaine << "   ok: " << ok;
	 SortValueChanged(value);
	 ui->SortValueSpinBox->setValue(value);
}


//-----------------------------------------------------------------------------
// SLOT SortValueChanged
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SortValueChanged(double value)
{
	 QString chaine;

	 chaine.setNum(value + 0.0001);		 // always decimal point
	 int pointPos = chaine.indexOf('.');
	 switch (pointPos)
	 {
		  case 1:
				if (ui->SortValueSpinBox->decimals() != 2)
					 ui->SortValueSpinBox->setDecimals(2);
				if (ui->SortValueSpinBox->singleStep() != 0.01)
					 ui->SortValueSpinBox->setSingleStep(0.01);
				break;
		  case 2:
			  if (ui->SortValueSpinBox->decimals() != 1)
					ui->SortValueSpinBox->setDecimals(1);
			  if (ui->SortValueSpinBox->singleStep() != 0.1)
					ui->SortValueSpinBox->setSingleStep(0.1);
				break;
		  case 3:
			  if (ui->SortValueSpinBox->decimals() != 0)
					ui->SortValueSpinBox->setDecimals(0);
			  if (ui->SortValueSpinBox->singleStep() != 1.0)
					ui->SortValueSpinBox->setSingleStep(1.0);
				break;
	 }
}


//-----------------------------------------------------------------------------
// SLOT closeParamSortingWidget
//-----------------------------------------------------------------------------
void AU2011_MainWindow::closeParamSortingWidget(void)
{
	 ui->SortingWidget->hide();
}


//-----------------------------------------------------------------------------
// SLOT validateParamSortingWidget
//-----------------------------------------------------------------------------
void AU2011_MainWindow::validateParamSortingWidget(void)
{
    int index;
	 QString chaine;
     QChar mu = 0x3bc;  // mu

	 chaineSort = trUtf8("  Sorting value:   ","  Valeur de tri:   ");
	 index = ui->SortKindComboBox->currentIndex();
	 switch(index)
	 {
		  case 0:
				SortKind = R_KIND;
				chaineSort.append( "R   ");
				break;
		  case 1:
				SortKind = L_KIND;
				chaineSort.append("L   ");
				break;
		  case 2:
				SortKind = C_KIND;
				chaineSort.append("C   ");
				break;
	 }
	 settings->setValue("SortingWidget/SortKindIndex", index);
	 settings->setValue("SortingWidget/SortKind", SortKind);

	 SortValue = ui->SortValueSpinBox->value();
	 chaine.setNum(SortValue);
	 chaineSort.append(chaine);

    index = ui->SortSuffixComboBox->currentIndex();
    switch(index)
    {
        case 0:
            SortSuffix = 1e-12;
				chaineSort.append('p');
				break;
        case 1:
            SortSuffix = 1e-9;
				chaineSort.append('n');
				break;
        case 2:
            SortSuffix = 1e-6;
				chaineSort.append(mu);
				break;
        case 3:
            SortSuffix = 1e-3;
				chaineSort.append('m');
				break;
        case 4:
            SortSuffix = 1.0;
				break;
        case 5:
            SortSuffix = 1e3;
				chaineSort.append('k');
				break;
        case 6:
            SortSuffix = 1e6;
				chaineSort.append('M');
				break;
	}
	settings->setValue("SortingWidget/SortSuffixIndex", index);
	settings->setValue("SortingWidget/SortSuffix", SortSuffix);

	SortValue *= SortSuffix;
	settings->setValue("SortingWidget/SortValue", SortValue);

	 if (SortKind == R_KIND)
		  chaineSort.append(Omega);
	 else if (SortKind == L_KIND)
		  chaineSort.append('H');
	 else if (SortKind == C_KIND)
		  chaineSort.append('F');

    index = ui->SortToleranceComboBox->currentIndex();
    switch(index)
	 {
        case 0:
            SortToleranceValue = 0.01;
				chaineSort.append("   1%");
				break;
        case 1:
            SortToleranceValue = 0.02;
				chaineSort.append("   2%");
				break;
        case 2:
            SortToleranceValue = 0.05;
				chaineSort.append("   5%");
				break;
        case 3:
            SortToleranceValue = 0.1;
				chaineSort.append("   10%");
				break;
        case 4:
            SortToleranceValue = 0.2;
				chaineSort.append("   20%");
				break;
    }
	settings->setValue("SortingWidget/SortToleranceIndex", index);
	settings->setValue("SortingWidget/SortTolerance", SortToleranceValue);

	settings->setValue("SortingWidget/ChaineSort", chaineSort);
	OkForSortAction->setEnabled(true);
	qDebug() << chaineSort;
    closeParamSortingWidget();
}


//-----------------------------------------------------------------------------
// SLOT sortAllowed
//-----------------------------------------------------------------------------
void AU2011_MainWindow::sortAllowed(void)
{
	 if (OkForSortAction->isChecked())
	 {
		  ui->SortPushButton->show();
		  ui->SortPushButton->setFocus();
		  barreEtat->showMessage(chaineSort, 0);
		  if (!onMeasurement)
			  RunStopMeasure();
	 }
	 else
	 {
		  ui->SortPushButton->hide();
		  barreEtat->showMessage("" , 10);
	 }
}


//-----------------------------------------------------------------------------
// SLOT setSortingParameters
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setSortingParameters(void)
{
    ui->SortingWidget->show();
	 ui->SortValueSpinBox->setAccelerated(true);

	 OkForSortAction->setEnabled(true);

	 if (settings->contains("SortingWidget/SortSuffixIndex"))
		  ui->SortSuffixComboBox->setCurrentIndex(settings->value("SortingWidget/SortSuffixIndex").toInt());
	 if (settings->contains("SortingWidget/SortToleranceIndex"))
		  ui->SortToleranceComboBox->setCurrentIndex(settings->value("SortingWidget/SortToleranceIndex").toInt());
	 if (settings->contains("SortingWidget/SortKindIndex"))
		  ui->SortKindComboBox->setCurrentIndex(settings->value("SortingWidget/SortKindIndex").toInt());
	 if (settings->contains("SortingWidget/SortValue"))
	 {
        double value = settings->value("SortingWidget/SortValue").toDouble();
        value /= settings->value("SortingWidget/SortSuffix").toDouble();
		ui->SortValueSpinBox->setValue(value);

	}
    if (onMeasurement)
        RunStopMeasure();

}


// *************  OFFSETS ***************
//-----------------------------------------------------------------------------
// SLOT SetBufferOffset
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetBufferOffset(void)
{
	port->write(HALT);
    qDebug() << "send HALT";
    onMeasurement = false;
    onOffsetAdjust = true;
	ui->runButton->setText(trUtf8("Start","Démarrer"));
    ui->runButton->setEnabled(false);
    setDelay(20);
	port->write(BUF_OFFSET);
    qDebug() << "send BUF_OFFSET";
	ui->offsetWidget->setWindowTitle(trUtf8("Buffer offset adjustment","Réglage offset du Buffer"));
	ui->offsetLabelLine1->setText(trUtf8("Use R75 trimpot","Régler l'offset par R75"));
	ui->offsetOKButton->setEnabled(true);
    ui->offsetOKButton->setVisible(false);
    ClearDataInWindow();
	ui->offsetWidget->show();
    setDelay(100);
}


//-----------------------------------------------------------------------------
// SLOT SetInputOffset_I
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetInputOffset_I(void)
{
    onOffsetAdjust = true;
    QString message = trUtf8("It's an automatic procedure","C'est une procédure automatique.");
    message.append('\n');
    message.append(trUtf8("OPEN-CIRCUIT inputs and press OK","OUVRIR LE CIRCUIT et validez"));
    int reponse = QMessageBox::question(this, "", message, QMessageBox::Ok | QMessageBox::Cancel);
    if (reponse != QMessageBox::Ok)
    {
        onOffsetAdjust = false;
        return;
    }
    ui->offsetLabelLine1->setText("");
    ui->offsetValueLabel->setText("");
    ui->offsetWidget->setWindowTitle(trUtf8("input offset adjustment","Réglage offset de l'entrée"));
    qDebug() << "send HALT";
    port->write(HALT);
    ClearDataInWindow();
    ui->runButton->setText(trUtf8("Start","Démarrer"));
    ui->runButton->setEnabled(false);
    onMeasurement = false;
    setDelay(200);
    qDebug() << "send INPUTF_OFFSET_I";
    port->write(INPUT_OFFSET_I);
    ui->offsetOKButton->setEnabled(true);
    ui->offsetOKButton->setVisible(false);
    ui->offsetWidget->show();
    Value_to_Print = 2.5;
    Display_Offset();
}


//-----------------------------------------------------------------------------
// SLOT SetInputOffset_V
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetInputOffset_V(void)
{
    onOffsetAdjust = true;
    QString message = trUtf8("It's an automatic procedure","C'est une procédure automatique.");
    message.append('\n');
    message.append(trUtf8("SHORT-CIRCUIT inputs and press OK","Faites un COURT-CIRCUIT à l'entrée puis validez"));

    int reponse = QMessageBox::question(this, "", message, QMessageBox::Ok | QMessageBox::Cancel);
    if (reponse != QMessageBox::Ok)
    {
        onOffsetAdjust = false;
        return;
    }
    ui->offsetLabelLine1->setText("");
    ui->offsetValueLabel->setText("");
    ui->offsetWidget->setWindowTitle(trUtf8("input offset adjustment","Réglage offset de l'entrée"));
    qDebug() << "send HALT";
    port->write(HALT);
    ClearDataInWindow();
    ui->runButton->setText(trUtf8("Start","Démarrer"));
    ui->runButton->setEnabled(false);
    onMeasurement = false;
    setDelay(200);
    qDebug() << "send INPUTF_OFFSET_V";
    port->write(INPUT_OFFSET_V);
    ui->offsetOKButton->setEnabled(true);
    ui->offsetOKButton->setVisible(false);
    ui->offsetWidget->show();
    Value_to_Print = 2.5;
    Display_Offset();
}


//-----------------------------------------------------------------------------
// SLOT SetSinusOffset
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetSinusOffset(void)
{
    onOffsetAdjust = true;

	QString message = trUtf8("It's an automatic procedure","C'est une procédure automatique.");
    message.append('\n');
    message.append(trUtf8("OPEN-CIRCUIT inputs and press OK","OUVRIR LE CIRCUIT d'entrée et validez"));

	int reponse = QMessageBox::question(this, "", message, QMessageBox::Ok | QMessageBox::Cancel);
	if (reponse != QMessageBox::Ok)
    {
        onOffsetAdjust = false;
		return;
    }
    ui->offsetLabelLine1->setText("");
    ui->offsetValueLabel->setText("");
    ui->offsetWidget->setWindowTitle(trUtf8("sinus-generator offset adjustment","Réglage offset du générateur sinus"));
    qDebug() << "send HALT";
	port->write(HALT);
    ClearDataInWindow();
	ui->runButton->setText(trUtf8("Start","Démarrer"));
    ui->runButton->setEnabled(false);
    onMeasurement = false;
	setDelay(200);
    qDebug() << "send SINUS_OFFSET";
	port->write(SINUS_OFFSET);
    ui->offsetOKButton->setEnabled(true);
    ui->offsetOKButton->setVisible(false);
    ui->offsetWidget->show();
    Value_to_Print = 2.5;
    Display_Offset();
}


//-----------------------------------------------------------------------------
// Display_Offset
//-----------------------------------------------------------------------------
void AU2011_MainWindow::Display_Offset(void)
{
	Convert_Value_to_String(3, WITH_SUFFIX, NO_KIND, false);
	ExpoBuf.append("V");
	ui->offsetValueLabel->setText(Buffer);
	ui->offsetUnityLabel->setText(ExpoBuf);
    if (fabs(Value_to_Print) < 0.005)
        ui->offsetOKButton->setVisible(true);
	else
		ui->offsetOKButton->setVisible(false);
}


//-----------------------------------------------------------------------------
// SLOT closeOffsetWidget
//-----------------------------------------------------------------------------
void AU2011_MainWindow::closeOffsetWidget(void)
{
    ui->offsetValueLabel->setText("");
    ui->offsetUnityLabel->setText("");
    ui->offsetOKButton->setVisible(false);
	port->write(OFFSET_OK);
    qDebug() << "send OFFSET_OK";
    ui->offsetWidget->hide();
    ui->runButton->setEnabled(true);
    onOffsetAdjust = false;
    setDelay(100);
}


// *************  PGA or INPUT PHASE ***************
//-----------------------------------------------------------------------------
// SLOT SetInputPhase
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetInputPhase(void)
{
     bool ok = (onMeasurement  && (Range_value > 2) && (Range_value <7) && (dac_U == dac_I) && (FreqValue == 10000));
     if (!ok)
     {
         QString message = trUtf8("The necessary conditions are not met","Les conditions nécessaires ne sont pas remplies");
         if (!onMeasurement)
         {
            message.append('\n');
            message.append(trUtf8(" - You must start the measurement"," - Vous devez démarrer la mesure"));
         }
         if (FreqValue != 10000)
         {
            message.append('\n');
            message.append(trUtf8(" - The frequency must be 10 kHz"," - La fréquence doit être 10kHz"));
         }
         if ((Range_value < 3 || Range_value > 6) && onMeasurement)
         {
            message.append('\n');
            message.append(trUtf8(" - The allowable Ranges are 3 to 6"," - Les Gammes autorisées sont 3 à 6"));
         }
         if (dac_U != dac_I && onMeasurement)
         {
            message.append('\n');
            message.append(trUtf8(" - U and I levels must be equal"," - Les paliers U et I doivent êtres égaux"));
         }

         this->setEnabled(false);   // deleting parent...
         QMessageBox::warning(this, "",message);
         this->setEnabled(true);
         return;
     }

    RunStopMeasure();
	ui->phaseAdjustWidget->setWindowTitle(
				trUtf8("Input phase adjustements","Réglage des phases d'entrée"));
    ui->phiValidationButton->setEnabled(true);
    ui->phiCancelButton->setEnabled(true);
    ClearDataInWindow();
    setPhaseAdjustMultiplier();
    ui->phaseAdjustWidget->show();
    setDelay(100);
    qDebug() << "send GO_PHI";
	port->write(GO_PHASE_ADJ);
    setDelay(20);
    onInputPhaseAdjust = true;
    QTimer::singleShot (500, this, SLOT(RunStopMeasure()));
}


//-----------------------------------------------------------------------------
// SLOT SetPGAPhase
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetPGAPhase(void)
{
     bool ok = (onMeasurement  && (Range_value < 3) && (dac_U == dac_I) && (FreqValue == 10000));
     if (!ok)
     {
         QString message = trUtf8("The necessary conditions are not met","Les conditions nécessaires ne sont pas remplies");
         if (!onMeasurement)
         {
            message.append('\n');
            message.append(trUtf8(" - You must start the measurement"," - Vous devez démarrer la mesure"));
         }
         if (FreqValue != 10000)
         {
            message.append('\n');
            message.append(trUtf8(" - The frequency must be 10 kHz"," - La fréquence doit être 10kHz"));
         }
         if (Range_value > 2 && onMeasurement)
         {
            message.append('\n');
            message.append(trUtf8(" - The allowable Ranges are 1 and 2"," - Les Gammes autorisées sont 1 et 2"));
         }
         if (dac_U != dac_I && onMeasurement)
         {
            message.append('\n');
            message.append(trUtf8(" - U and I levels must be equal"," - Les paliers U et I doivent êtres égaux"));
         }

         this->setEnabled(false);   // deleting parent...
         QMessageBox::warning(this, "",message);
         this->setEnabled(true);
         return;
     }

    RunStopMeasure();
    ui->phaseAdjustWidget->setWindowTitle(
				trUtf8("PGA phase adjustements","Réglage phases PGA"));
	ui->phiValidationButton->setEnabled(true);
    ui->phiCancelButton->setEnabled(true);
    ClearDataInWindow();
    setPhaseAdjustMultiplier();
	ui->phaseAdjustWidget->show();
    setDelay(100);
    qDebug() << "send GO_PGA";
	port->write(GO_PGA_ADJ);
    setDelay(20);
    onPGAPhaseAdjust = true;
    QTimer::singleShot (200, this, SLOT(RunStopMeasure()));
}


//-----------------------------------------------------------------------------
// SLOT setPhaseAdjustMultiplier
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setPhaseAdjustMultiplier(void)
{
	if (ui->radioButton11->isChecked())
        PhiMultiplier = 1.1;
    else if (ui->radioButton101->isChecked())
        PhiMultiplier = 1.01;
    else if (ui->radioButton1001->isChecked())
        PhiMultiplier = 1.001;
    else if (ui->radioButton10001->isChecked())
        PhiMultiplier = 1.0001;
}


//-----------------------------------------------------------------------------
// SLOT setPlusMinusPhaseAdjustMultiplier
//-----------------------------------------------------------------------------
void AU2011_MainWindow::setPlusMinusPhaseAdjustMultiplier(int state)
{
    char mystr[30];

	if ( (state == Qt::Checked && PhiData > 0.0) || (state == Qt::Unchecked && PhiData < 0.0))
    {
        PhiData = - PhiData;
        sprintf(mystr, "PHIDAT %f\n", PhiData);
        qDebug() << "send " << mystr;
        port->write(mystr);
        Display_PhiData();
	}
}


// *************  PHI ***************
//-----------------------------------------------------------------------------
// Display_PhiData
//-----------------------------------------------------------------------------
void AU2011_MainWindow::Display_PhiData(void)
{
	Value_to_Print = PhiData;
	Convert_Value_to_String(3, WITH_SUFFIX, NO_KIND, false);
	ui->phiDataValueLabel->setText(Buffer);
	ui->phiDataUnityLabel->setText(ExpoBuf);
}


//-----------------------------------------------------------------------------
// SLOT increasePhiDataValue
//-----------------------------------------------------------------------------
void AU2011_MainWindow::increasePhiDataValue(void)
{
    char mystr[30];

    PhiData *= PhiMultiplier;
    sprintf(mystr, "PHIDAT %f\n", PhiData);
    qDebug() << "send " << mystr;
    port->write(mystr);
	Display_PhiData();
}


//-----------------------------------------------------------------------------
// SLOT decreasePhiDataValue
//-----------------------------------------------------------------------------
void AU2011_MainWindow::decreasePhiDataValue(void)
{
    char mystr[30];

    PhiData /= PhiMultiplier;
    sprintf(mystr, "PHIDAT %f\n", PhiData);
    qDebug() << "send " << mystr;
    port->write(mystr);
	Display_PhiData();
}


//-----------------------------------------------------------------------------
// SLOT validatePhiWidget
//-----------------------------------------------------------------------------
void AU2011_MainWindow::validatePhiWidget(void)
{
    if (onInputPhaseAdjust)
    {
        qDebug() << "send SAVPHI";
		port->write(SAVE_PHASE);
    }
    else if (onPGAPhaseAdjust)
    {
        qDebug() << "send SAVPGA";
		port->write(SAVE_PGA);
    }
	closePhiWidget();
}


//-----------------------------------------------------------------------------
// SLOT closePhiWidget
//-----------------------------------------------------------------------------
void AU2011_MainWindow::closePhiWidget(void)
{
	onInputPhaseAdjust = false;
    onPGAPhaseAdjust = false;
    qDebug() << "send EXPHPG";
	port->write(EXIT_PHI_PGA);
	ui->phaseAdjustWidget->hide();
}


// *************  RESISTORS ***************
//-----------------------------------------------------------------------------
// SLOT SetResistors
//-----------------------------------------------------------------------------
void AU2011_MainWindow::SetResistors(void)
{
	port->write(HALT);
    qDebug() << "send HALT";
    onMeasurement = false;
    onResistorsAdjust = false; // don't check value first time
	ui->runButton->setText(trUtf8("Start","Démarrer"));
    ui->runButton->setEnabled(false);
    ui->resistorValidationButton->setEnabled(true);
    ui->resistorCancelButton->setEnabled(true);
    ClearDataInWindow();
    ui->senseResistorsWidget->show();
    QTimer::singleShot (100, this, SLOT(changeSelectedResistor()));
}


//-----------------------------------------------------------------------------
// SLOT changeSelectedResistor
//-----------------------------------------------------------------------------
void AU2011_MainWindow::changeSelectedResistor(void)
{
    if (onResistorsAdjust)
    {
        double spinBoxValue = ui->resistorValueSpinBox->value();

        bool okmessage = ((spinBoxValue > initialRsenseValue * 1.00002)
											|| (spinBoxValue < initialRsenseValue * 0.99998));

        if (okmessage)
         {
            QMessageBox msgBox;

			msgBox.setText(trUtf8("Resistor value modified","La valeur de la résistance a été modifiée."));
			msgBox.setInformativeText(trUtf8("Save new value?","Voulez vous sauvegarder la nouvelle valeur?"));
            msgBox.setIcon(QMessageBox::Question);
            msgBox.setStandardButtons(QMessageBox::Save | QMessageBox::Discard);
            msgBox.setDefaultButton(QMessageBox::Save);
            int reponse = msgBox.exec();
            if (reponse == QMessageBox::Save)
                validateResistorWidget();
         }
    }
    else
        onResistorsAdjust = true;

    if (ui->radioButton100k->isChecked())
    {
        rsenseButtonSelected = 4;

        ui->resistorValueSpinBox->setRange(99000, 101000);
        ui->resistorValueSpinBox->setDecimals(0);
        ui->resistorValueSpinBox->setSingleStep(2);
        qDebug() << "send QUERY_R_100K";
		port->write(QUERY_R_100K);
    }
    else if (ui->radioButton10k->isChecked())
    {
        rsenseButtonSelected = 3;
        ui->resistorValueSpinBox->setRange(9900.0, 10100.0);
        ui->resistorValueSpinBox->setDecimals(1);
        ui->resistorValueSpinBox->setSingleStep(0.2);
        qDebug() << "send QUERY_R_10K";
		port->write(QUERY_R_10K);
    }
    else if (ui->radioButton1k->isChecked())
    {
        rsenseButtonSelected = 2;
        ui->resistorValueSpinBox->setRange(990.00, 1010.00);
        ui->resistorValueSpinBox->setDecimals(2);
        ui->resistorValueSpinBox->setSingleStep(0.02);
        qDebug() << "send QUERY_R_1K";
		port->write(QUERY_R_1K);
    }
    else if (ui->radioButton100R->isChecked())
    {
        rsenseButtonSelected = 1;
        ui->resistorValueSpinBox->setRange(99.000, 101.000);
        ui->resistorValueSpinBox->setDecimals(3);
        ui->resistorValueSpinBox->setSingleStep(0.002);
        qDebug() << "send QUERY_R_100R";
		port->write(QUERY_R_100R);
    }
	//setDelay(20);
}


//-----------------------------------------------------------------------------
// SLOT validateResistorWidget
//-----------------------------------------------------------------------------
void AU2011_MainWindow::validateResistorWidget(void)
{
    char message[MESSAGE_SIZE + 1];
    char valueR[12];

    double resistor = ui->resistorValueSpinBox->value();

    if (rsenseButtonSelected == 4)
    {
        strcpy(message, R_100K);
        sprintf(valueR, "%1.1f", resistor);
    }
    else if (rsenseButtonSelected == 3)
    {
        strcpy(message, R_10K);
        sprintf(valueR, "%1.2f", resistor);
    }
    else if (rsenseButtonSelected == 2)
    {
        strcpy(message, R_1K);
        sprintf(valueR, "%1.3f", resistor);
    }
    else if (rsenseButtonSelected == 1)
    {
        strcpy(message, R_100R);
        sprintf(valueR, "%1.4f", resistor);
    }
    QString theMessage = trUtf8("Save value","Sauvegarder la valeur");
    theMessage.append("\n");
    theMessage.append(valueR);
    theMessage.append(" ");
    theMessage.append(Omega);
    this->setEnabled(false);
    int reponse =  QMessageBox::question(this, "",theMessage, QMessageBox::Yes | QMessageBox::No);
    this->setEnabled(true);
    if (reponse != QMessageBox::Yes)
        return;

    if (strlen(message) + strlen(valueR)  <=  MESSAGE_SIZE) // prevent overflow
    {
        strcat(message, valueR);
        strcat(message,"\n");
        qDebug() << message;
        initialRsenseValue = resistor;
        port->write(message);
        setDelay(2);
    }
    //closeResistorWidget();
}


//-----------------------------------------------------------------------------
// SLOT closeResistorWidget
//-----------------------------------------------------------------------------
void AU2011_MainWindow::closeResistorWidget(void)
{
    ui->runButton->setEnabled(true);
    ui->senseResistorsWidget->hide();
    onResistorsAdjust = false;
}


//-----------------------------------------------------------------------------
// SLOT Trim
//-----------------------------------------------------------------------------
void AU2011_MainWindow::Trim(void)
{
     onMeasurement = false;
	 ui->runButton->setText(trUtf8("Start","Démarrer"));
     ui->runButton->setEnabled(false);
     ui->trimPushButton->setEnabled(false);
     ui->FreqGroupBox->setEnabled(false);
     ui->ModeGroupBox->setEnabled(false);
     onTrimAdjust = true;
     if (ZValue < 10.0)
     {
         qDebug() << "send S_TRIM_SHORT";
		 port->write(S_TRIM_SHORT);
         trimOperation = TRIM_SHORT;
     }
     else if (ZValue > 1e6)
	 {
         qDebug() << "send S_TRIM_OPEN";
		 port->write(S_TRIM_OPEN);
         trimOperation = TRIM_OPEN;
     }
}


//-----------------------------------------------------------------------------
// ProcessAllParameters
//-----------------------------------------------------------------------------
void AU2011_MainWindow::ProcessAllParameters(void)
{
    double pulsation = TWO_PI * FreqValue;
    double Rs = fabs(RsValue);
    double Xs = fabs(XsValue);

    // due to trim over-compensation, look for "normal"
    //      very low values (C in 0 pF range, L in few nH range)
    if (isCapacitive && Rs > 1e6 && Xs > 1e6)
    {
        if (XsValue > 0.0)
            XsValue = - XsValue;

    }
    else if (!isCapacitive && Rs < 0.05 && Xs < 0.05)
    {
        if (XsValue < 0.0)
            XsValue = - XsValue;
    }
    if (RsValue < 0.0)
        RsValue = - RsValue;

    PhiValue = atan(XsValue/RsValue);
    if (isCapacitive && PhiValue > 0.0)
        PhiValue = - PhiValue;
    QValue = fabs(XsValue/RsValue);
	if (QValue > 1.0)
		AutoParameter = REACTIVE_CIRCUIT;			// L or C is dominant param.
    else
		AutoParameter = RESISTIVE_CIRCUIT;			// R is dominant param.

    ZValue = sqrt(XsValue * XsValue + RsValue * RsValue);
    SP_coeff = 1 + 1 / (QValue * QValue);
    if (isCapacitive)
        CsValue = -1 / pulsation / XsValue;
    else
        LsValue = XsValue / pulsation;
    if (AutoModel == 0)                     // Auto mode
    {
        if (ZValue > Z_CHANGE_MODEL)
            SP_mode = 1;                    // parallel
        else
            SP_mode = 0;                    // series
    }
    else
        SP_mode = AutoModel - 1;            // L or C

	if ((ZValue < 10.0) || (ZValue > 1e6))
        ui->trimPushButton->setEnabled(true);
    else
        ui->trimPushButton->setEnabled(false);
}

//-----------------------------------------------------------------------------
// Display_Primary_Value
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : kind of component
//
// display the primary value with prefix and unity
// display flashing LED
//
void AU2011_MainWindow::Display_Primary_Value(void)
{
    uchar accuracy = 3 + 3 * MaxPrecision;

#if defined (Q_OS_WIN)
    int ms = 50;
#elif defined(Q_OS_LINUX)
	int ms = 50;
#elif defined(Q_OS_MACX)
    int ms = 50;
#endif

	PrimaryValue = Value_to_Print;
    Convert_Value_to_String(accuracy, WITH_SUFFIX, primaryKind, FALSE);
	switch (primaryKind)
	{
		case R_KIND:
			ExpoBuf.append(Omega);
			break;
		case L_KIND:
			ExpoBuf.append("H");
			break;
		case C_KIND:
			ExpoBuf.append("F");
			break;
		default:;
	}
	if (ui->ledCheckBox->isChecked())
    {
        ui->ledLabel->setStyleSheet("color: green");
        ui->ledLabel->show();
		QTimer::singleShot (ms, this, SLOT(switchLedOff()));
	}
	ui->PrimaryValueLabel->setText(Buffer);
	ui->PrimaryUnityLabel->setText(ExpoBuf);
}

//-----------------------------------------------------------------------------
// Display_Secondary_Value
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : kind of component
//
// display the secondary value with prefix and unity, or display PT100 temperature
//
void AU2011_MainWindow::Display_Secondary_Value(void)
{
    uchar accuracy = 3 * MaxPrecision;

	bool okTemp = (RsValue >= 70.0) && (RsValue <= 300.0) && (FreqValue <= 1000) && ( QValue > Q_CHANGE_MODEL || QValue < (1 / Q_CHANGE_MODEL) );

	//qDebug() << "okTemp: " << okTemp;

	if (!okTemp)
		ui->PT100CheckBox->hide();
	else
		ui->PT100CheckBox->show();

	if ( QValue > Q_CHANGE_MODEL || QValue < (1 / Q_CHANGE_MODEL) )
	{
		if (okTemp && ui->PT100CheckBox->isChecked())
		{
            Value_to_Print = PT100_Temperature(RsValue);
            QString value;
            value.setNum(Value_to_Print,'f',1);
            strcpy(Buffer, value.toAscii());
            ExpoBuf = "";
            ExpoBuf.append(Qt::Key_degree);
			ExpoBuf.append('C');
			circuitKind = PLAIN;
			secondaryKind = NO_KIND;
			primaryKind = T_KIND;
		}
		else
		{
            strcpy(Buffer, "");
			ExpoBuf = "";
			circuitKind = PLAIN;
			secondaryKind = NO_KIND;
		}
	}
	else
	{
		Convert_Value_to_String(accuracy, WITH_SUFFIX, secondaryKind, TRUE);
		switch (secondaryKind)
		{
			case R_KIND:
				ExpoBuf.append(Omega);
				break;
			case L_KIND:
				ExpoBuf.append("H");
				break;
			case C_KIND:
				ExpoBuf.append("F");
				break;
			default:;
		}
	}
	ui->SecondaryValueLabel->setText(Buffer);
	ui->SecondaryUnityLabel->setText(ExpoBuf);
}

//-----------------------------------------------------------------------------
// Process_Circuit_Xs
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// L or C as primary, and R as secondary
//
void AU2011_MainWindow::ProcessCircuitXs(void)
{
	// ---------- Primary value
	if (isCapacitive)
	{
		if (SP_mode == 0)										// serie
		{
			Value_to_Print = CsValue;
			circuitKind = SERIES;
		}
		else
		{
			Value_to_Print = CsValue / SP_coeff;		// "Cp "
			circuitKind = PARALLEL;
		}
		primaryKind = C_KIND;
	}
   else
    {
        if (SP_mode == 0)                                       // serie
        {
            Value_to_Print = LsValue;
            circuitKind = SERIES;
        }
        else
        {
            Value_to_Print = LsValue * SP_coeff;        // "Lp "
            circuitKind = PARALLEL;
        }
        primaryKind = L_KIND;
    }

    Display_Primary_Value();

    // ---------- Secondary value
	if (SP_mode == 0)                                            // serie
        Value_to_Print = RsValue;
    else
        Value_to_Print = RsValue * (1 + QValue * QValue);     // "Rp "
    secondaryKind = R_KIND;
    Display_Secondary_Value();
}

//-----------------------------------------------------------------------------
// Process_Circuit_Rs
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Rs and Ls, or Rp and Cp
//
void AU2011_MainWindow::ProcessCircuitRs(void)
{
	// ---------- Primary value
	if (SP_mode == 0)												// serie
		Value_to_Print = RsValue;
	else
		Value_to_Print = RsValue * (1 + QValue * QValue);   // "Rp "
	primaryKind = R_KIND;
	Display_Primary_Value();

    // ---------- Secondary value
    if (isCapacitive)
    {
		if (SP_mode == 0)                                           // serie
        {
            Value_to_Print = CsValue;
            circuitKind = SERIES;
        }
        else
        {
			Value_to_Print = CsValue / SP_coeff;            // "Cp "
            circuitKind = PARALLEL;
        }
        secondaryKind = C_KIND;
    }
    else
    {
		if (SP_mode == 0)                                           // serie
        {
            Value_to_Print = LsValue;
            circuitKind = SERIES;
        }
        else
        {
			Value_to_Print = LsValue * SP_coeff;            // "Lp "
            circuitKind = PARALLEL;
        }
        secondaryKind = L_KIND;
    }
    Display_Secondary_Value();
}

//-----------------------------------------------------------------------------
// Display_Frequency
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// display the frequency
//
void AU2011_MainWindow::Display_Frequency(void)
{
	QString buf ="";

	switch (FreqValue)
	{
		case 100:
			buf = "100 Hz";
			break;
		case 120:
			buf = "120 Hz";
			break;
		case 1000:
			buf = " 1 kHz";
			break;
		case 10000:
			buf = "10 kHz";
			break;
	}
	ui->FreqValueLabel->setText(buf);
} // Display_Frequency


//-----------------------------------------------------------------------------
// Display_Q_D
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// display parameter Q or D
//
void AU2011_MainWindow::Display_Q_D (void)
{
	uchar i;
	QString str;
	bool inverse = (ui->QDcheckBox->checkState () == Qt::Checked);

	if (inverse)
	{
		if (AutoParameter && isCapacitive)
		{
			Value_to_Print = QValue;
			str = "Q";
		}
		else
		{
			Value_to_Print = 1/QValue;
			str = "D";
		}
	}
	else
	{
		if (AutoParameter && isCapacitive)
		{
			Value_to_Print = 1/QValue;
			str = "D";
		}
		else
		{
			Value_to_Print = QValue;
			str = "Q";
		}
	}

	if (Value_to_Print < 1)
		i = 1;
	else if (Value_to_Print < 10)
		i = 2;
	else if (Value_to_Print < 100)
		i = 3;
	else if (Value_to_Print < 1000)
		i = 4;
	else if (Value_to_Print < 10000)
		i = 5;
	else
		i = 0;

	if (i)
        sprintf(Buffer, Table_format_QD[i-1], Value_to_Print);
	else
        strcpy(Buffer, ">10k");

    // internationalization
    setLocalizedDecimalPoint(Buffer);

    ui->QD_ValueLabel->setText(Buffer);
	ui->label_QD->setText(str);
} // Display_Q_D


//-----------------------------------------------------------------------------
// Display_PHI
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// display parameter PHI
//
void AU2011_MainWindow::Display_PHI(void)
{
    uchar format;

	Value_to_Print = PhiValue * 180 / PI ;	// degree
    if (onInputPhaseAdjust || onPGAPhaseAdjust)
		format = 6;
	else
		format = 3 * MaxPrecision;

	Convert_Value_to_String(format, WITH_SUFFIX, NO_KIND, FALSE);

	ExpoBuf.append(" ");
    ExpoBuf.append(Qt::Key_degree);
	ui->PhiValueLabel->setText(Buffer);
	ui->PhiUnityLabel->setText(ExpoBuf);
} // Display_PHI


//-----------------------------------------------------------------------------
// Display_Z
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// display parameter Z
//
void AU2011_MainWindow::Display_Z(void)
{
    uchar format = 3 + 3 * MaxPrecision;

	Value_to_Print = ZValue;

	Convert_Value_to_String(format, WITH_SUFFIX, R_KIND, TRUE);

	ExpoBuf.append(Omega);
	ui->Z_ValueLabel->setText(Buffer);
	ui->Z_UnityLabel->setText(ExpoBuf);
} // Display_Z


//-----------------------------------------------------------------------------
// Display_VxRs
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// display selected parameter in Preference pane
//
void AU2011_MainWindow::Display_VxRs(void)
{

	uchar format = 3 * MaxPrecision;
	int x = ui->DisplayUIComboBox->currentIndex();

	if (x == DISPLAY_U_I)
	{
		Value_to_Print = VppValue / (TWO_SQRT_2);
		Convert_Value_to_String(format, WITH_SUFFIX, NO_KIND, TRUE);
		ExpoBuf.append("V");
		ui->VxRsLabel->setText("Ux");
	}
	else if (x == DISPLAY_R_X)
	{
		Value_to_Print = RsValue;
		format += 3;
        Convert_Value_to_String(format, WITH_SUFFIX, R_KIND, FALSE);
		ExpoBuf.append(Omega);
		ui->VxRsLabel->setText("Rs");
	}
	else if (x == DISPLAY_ADC)
	{
		Value_to_Print = ADC_Vpp;
		Convert_Value_to_String(format, WITH_SUFFIX, R_KIND, TRUE);
		ExpoBuf.append("V");
		ui->VxRsLabel->setText("ADC U");
	}
	ui->VxRsValueLabel->setText(Buffer);
	ui->VxRsUnityLabel->setText(ExpoBuf);
} // Display_VxRs


//-----------------------------------------------------------------------------
// Display_IxXs
//-----------------------------------------------------------------------------

// display selected parameter in Preference pane
//
void AU2011_MainWindow::Display_IxXs(void)
{
	uchar format = 3 * MaxPrecision;
	int x = ui->DisplayUIComboBox->currentIndex();

	if (x == DISPLAY_U_I)
	{
		Value_to_Print = IppValue / (TWO_SQRT_2);
		Convert_Value_to_String(format, WITH_SUFFIX, NO_KIND, TRUE);
		ExpoBuf.append("A");
		ui->IxXsLabel->setText("Ix");
	}
	else if (x == DISPLAY_R_X)
	{
		Value_to_Print = XsValue;
		format += 3;
        Convert_Value_to_String(format, WITH_SUFFIX, R_KIND, FALSE);
		ExpoBuf.append(Omega);
		ui->IxXsLabel->setText("Xs");
	}
	else if (x == DISPLAY_ADC)
	{
		Value_to_Print = ADC_Ipp;
		Convert_Value_to_String(format, WITH_SUFFIX, R_KIND, TRUE);
		ExpoBuf.append("V");
		ui->IxXsLabel->setText("ADC I");
	}

	ui->IxXsValueLabel->setText(Buffer);
	ui->IxXsUnityLabel->setText(ExpoBuf);
} // Display_IxXs


//-----------------------------------------------------------------------------
// Display_Ranges
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// display range & dac's index
//
void AU2011_MainWindow::Display_Ranges(void)
{
    QString str;

    if (ui->InfoGainCheckBox->isChecked())
         ui->GainFreqFrame->show();
    else
         ui->GainFreqFrame->hide();

    str.setNum(Range_value);
    ui->RangeValueLabel->setText(str);
    if (allow_DAC8811_32)
    {
        str.setNum(dac_U);
        ui->DAC_U_ValueLabel->setText(str);
        str.setNum(dac_I);
        ui->DAC_I_ValueLabel->setText(str);
    }
    else
    {
        str.setNum(dac_U, 16);
        ui->DAC_U_ValueLabel->setText(str.toUpper());
        str.setNum(dac_I, 16);
        ui->DAC_I_ValueLabel->setText(str.toUpper());
    }

    switch (Range_value)
    {
        case 1:
        case 8:
            ui->RangeValueLabel->setStyleSheet("color: red");
            break;
        case 2:
        case 7:
            ui->RangeValueLabel->setStyleSheet("color: magenta");
            break;
        default:
            ui->RangeValueLabel->setStyleSheet("color: green");
            break;
    }
} // Display_Ranges



//-----------------------------------------------------------------------------
// Display_Accuracy
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
void AU2011_MainWindow::Display_Accuracy(void)
{
    double error_value = Gain_Error() * 100;
    error_value = ceil(error_value) / 100;      // round up error_value
    if (Range_value > 2 && Range_value < 7)
        ui->ErrorCheckBox->hide();
    else
        ui->ErrorCheckBox->show();
	QString ErrorStr;
    ErrorStr.setNum(error_value, 'f', 2);
	ui->ErrorValueLabel->setText(ErrorStr);
}


//-----------------------------------------------------------------------------
// Display_Symboles
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
void AU2011_MainWindow::Display_Symboles(void)
{
	char choice;
	QString PNG_file;

	if (circuitKind == PLAIN)
	{
		switch (primaryKind) {
		case R_KIND:
			PNG_file = "R_seul.png";
			choice = 1;
			break;
		case C_KIND:
			PNG_file = "C_seul.png";
			choice = 2;
			break;
		case L_KIND:
			PNG_file = "L_seul.png";
			choice = 3;
			break;
		case T_KIND:
			PNG_file = "RT_seul.png";
			choice = 20;
			break;
		default:
			PNG_file = "";
			choice = 0;
		}
	}
	else if (circuitKind == SERIES)
	{
		switch (primaryKind) {
		case R_KIND:
			if (secondaryKind == C_KIND)
			{
				PNG_file = "RC_serie.png";
				choice = 4;
			}
			else
			{
				PNG_file = "RL_serie.png";
				choice = 5;
			}
			break;
		case C_KIND:
			PNG_file = "CR_serie.png";
			choice = 6;
			break;
		case L_KIND:
			PNG_file = "LR_serie.png";
			choice = 7;
			break;

		default:
			PNG_file = "";
			choice = 0;
		}
	}
	else if (circuitKind == PARALLEL)
	{
		switch (primaryKind) {
		case R_KIND:
			if (secondaryKind == C_KIND)
			{
				PNG_file = "RC_para.png";
				choice = 8;
			}
			else
			{
				PNG_file = "RL_para.png";
				choice = 9;
			}
			break;
		case C_KIND:
			PNG_file = "CR_para.png";
			choice = 10;
			break;
		case L_KIND:
			PNG_file = "LR_para.png";
			choice = 11;
			break;
		default:
			PNG_file = "";
			choice = 0;
		}
	}

	if (choice != savedChoice)
	{
		//qDebug() << PNG_file;
		ui->SchematicLabel->setPixmap(QPixmap(":/AU2011_Projet/Resources/" + PNG_file));
		savedChoice = choice;
	}
} // Display_Symboles


//-----------------------------------------------------------------------------
// Gain_Error
//-----------------------------------------------------------------------------
//
// Return Value : the value (%)
// Parameters   : None
//
double AU2011_MainWindow::Gain_Error(void)
{
    double error_value = 0.0;

    if (dac_U != dac_I)
    {
        error_value = 1 / pow(Exx_Coeff, dac_U);
        error_value += 1 / pow(Exx_Coeff, dac_I);
        error_value *= 100 * GMax_DAC / 65535;
    }

    if (Range_value == 1 || Range_value == 8)
    {
        if (ui->ErrorCheckBox->isChecked())
            error_value += PGA_GAIN_ERROR_100_MAX;
        else
            error_value += PGA_GAIN_ERROR_100;
    }
    else if (Range_value == 2 || Range_value == 7)
    {
        if (ui->ErrorCheckBox->isChecked())
            error_value += PGA_GAIN_ERROR_10_MAX;
        else
            error_value += PGA_GAIN_ERROR_10;
    }

    return error_value;
    //qDebug() << "error: " << error_value ;
}


//-----------------------------------------------------------------------------
// Convert_Value_to_String
//-----------------------------------------------------------------------------
//
// Return Value :	Number of character in Buffer
// Parameters   :	accuracy : 0 for 3 digits, 3 for 4 digits, 6 for 5 digits
//							suffix : add suffix or not
//							Comp_Kind : kind of componant to display value (R, L, C)
//							adjust_format : if true, modify accuracy if needed
//
// Convert double 'Value_to_Print' to formated string in 'Buffer'
//		according accuracy and kind of componant
// Put suffix (f, p, n, u, m, k, M, G, T) in 'Expo' if boolean 'suffix' is TRUE
//
//
char AU2011_MainWindow::Convert_Value_to_String (
		char accuracy, bool suffix, char Comp_Kind, bool adjust_format)
{
	char  expo;
	int format;
	char len;
    double f1;
	double int_part;
	bool  lower_1;
	bool  negatif;
	bool special_case = false;

    negatif = (Value_to_Print < 0.0);

	if ((negatif) && (accuracy > 3) && (adjust_format))
		accuracy -=3;

    f1 = fabs(Value_to_Print);

    // Deal with specific cases
	if ((Comp_Kind == R_KIND) && (f1 > 1.00e9))		// 1 Gohm
	{
		if (negatif)
            strcpy(Buffer, "< - 1");
		else
            strcpy(Buffer, " > 1");
		ExpoBuf = "G";
		special_case = true;
	}
	else if ((Comp_Kind == C_KIND) && (f1 > 0.1))		// 100mF
	{
        strcpy(Buffer, ">100");
		ExpoBuf = "m";
		special_case = true;
	}
	else if ((Comp_Kind == L_KIND) && (f1 > 100))		// 100H
	{
        strcpy(Buffer, ">100");
		ExpoBuf = "";
		special_case = true;
	}

    if ( special_case)  // all is done!
	{
		len = strlen(Buffer);
		return len;
	}


	if (f1 != 0.0)						// 0.0 is special case!
	{
		lower_1 = (f1 < 1.0);			// true if value < 1.0
		f1 = log10(f1);
		f1 = modf (f1, &int_part);		// f1 is fractionnal part of log10
										// int_part is integer part
		f1 = pow(10,f1);				// f1 is formated value x.xxx to xxx.x
    }
	else
		int_part = 1.0;

    // improved accuracy for calibration with 0.01% 100Ω, 1kΩ, 10kΩ and 100kΩ resistor and adjustAccessCheckBox is checked
    if (ui->adjustAccessCheckBox->isChecked() && f1 > 0.999 && f1 < 1.001)
        special_case = true;

	if ((!lower_1) && (f1 > 9.0) && (accuracy == 6) && (adjust_format))
		accuracy = 3;

   expo = (char)fmod(int_part, 3.0);

   if (lower_1)								// value < 1.0
   {
       if ( (Comp_Kind == C_KIND) && (int_part <= -9) )				// display in picoFarad
	   {
           format = 2;                  // 2 digits after the decimal point
		   if (int_part == -9)
           {
			   f1 *= 1000.0;
               format = 1;              // 1 digits after the decimal point
           }
		   else if (int_part == -10)
			   f1 *= 100.0;
		   else if (int_part == -11)
			   f1 *= 10.0;
		   else if (int_part == -12)
			   ;
		   else if (int_part == -13)
			   f1 *= 0.1;
		   else if (int_part == -14)
			   f1 *= 0.01;
		   else if (int_part == -15)
			   f1 *= 0.001;
		   else
			   f1 = 0.0;
		   int_part = -12;
       }
       else if ( (Comp_Kind == L_KIND) && (int_part <= -6) )		// display in nanoHenri
       {
           format = 1;                  // 1 digits after the decimal point
           if (int_part == -6)
               f1 *= 1000.0;
            else if (int_part == -7)
               f1 *= 100.0;
           else if (int_part == -8)
               f1 *= 10.0;
           else if (int_part == -9)
               ;
           else if (int_part == -10)
               f1 *= 0.1;
           else if (int_part == -11)
               f1 *= 0.01;
           else if (int_part == -12)
               f1 *= 0.001;
           else
               f1 = 0.0;
           int_part = -9;
       }
       else if ( (Comp_Kind == R_KIND) && (int_part <= 0) )			// display in milliOhm
       {
           if (int_part == 0)           // 0.xxxx
		   {
			   f1 *=1000.0;
               format = 1;              // 1 digits after the decimal point
		   }
           else if (int_part == -1)    // 0.0xxxx
		   {
			   f1 *=100.0;
               format = 2;              // 2 digits after the decimal point
		   }
           else if (int_part == -2)    // 0.00xxxx
		   {
			   f1 *=10.0;
               format = 2;              // 2 digits after the decimal point
           }
           else if (int_part == -3)    // 0.000xxxx
               format = 1;              // 1 digits after the decimal point
		   else
           {
			   f1 = 0.0;
               format = 2;          // 0.00
           }
		   int_part = -3;
	   }
        else
        {
            switch (expo)
            {
                case 0:
                    f1 *= 1000;
                    int_part -= 3.0;
                    format = 0 + accuracy;
                    break;
                case -1:
                    f1 *= 100;
                    int_part -= 2.0;
                    format = 1 + accuracy;
                    break;
                case -2:
                    f1 *= 10;
                    int_part -= 1.0;
                    format = 2 + accuracy;
                    break;
            } // switch
        }
   } // if (lower_1)
   else if (!lower_1)                 				// value >= 1.0
   {
		switch (expo)
		{
			case 0:
				format = 2 + accuracy;
				break;
			case 1:
                f1 *= 10.0;
				int_part -= 1.0;
				format = 1 + accuracy;
				break;
			case 2:
                f1 *= 100.0;
				int_part -= 2.0;
				format = 0 + accuracy;
				break;
		}

		if (Comp_Kind == C_KIND)
			format = 0;
   }

   if ( special_case && format < 9)
       format++;

    sprintf (Buffer, Table_format[format], f1);
	len = strlen(Buffer);

    if (negatif)    // put '-' in front of Buffer
	{
		if (len == BUFFER_SIZE)
		{
			Buffer[BUFFER_SIZE - 1] = '\0';
			len--;
		}

		for (int i = len; i >= 0; i--)
		{
			Buffer[i+1] = Buffer[i];
		}
		Buffer[0] = '-';
		len++;
	}

    // internationalization
    setLocalizedDecimalPoint(Buffer);

     ExpoBuf = "";

	if (suffix == WITHOUT_SUFFIX)
		return len;

    switch ((char)int_part)          // put suffix
   {
       case -15:
         ExpoBuf = "f";
         break;
      case -12:
         ExpoBuf = "p";
         break;
      case -9:
         ExpoBuf = "n";
         break;
      case -6:
         ExpoBuf = Qt::Key_mu;
         break;
      case -3:
         ExpoBuf = "m";
         break;
        case 0:
         break;
      case 3:
         ExpoBuf = "k";
         break;
      case 6:
         ExpoBuf = "M";
         break;
       case 9:
         ExpoBuf = "G";
         break;
      case 12:
         ExpoBuf = "T";
         break;
    }
    return len;
} // Convert_Value_to_String


//-----------------------------------------------------------------------------
// PT100_Temperature()
//-----------------------------------------------------------------------------
//
// Return value: temperature for the PT100 probe
// Parameters: RTD value to convert
//
// temperature calculation by linear interpolation off two adjacents T_RTD[] values
//
double AU2011_MainWindow::PT100_Temperature(double RT)
{
    double rt = RT/10;
	uchar entier = (uchar)rt;
    if (entier > 39)    // entier + 1 out of T_RTD array[]
        return -273.16; // 0°K !!!...
    double frac_rt = rt - entier;
    double a = T_RTD[entier];
    double b = T_RTD[entier + 1];
    rt = a + frac_rt * (b - a);
    return rt;
}


//-----------------------------------------------------------------------------
// setDelay()
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters: delay needed in ms. max is 1000 ms (1s)
//
void AU2011_MainWindow::setDelay( uint i)
{
	if (i > 1000)
		i = 1000;
	if (i == 0)
		i++;
#ifdef Q_OS_WIN
	Sleep(i);	// ms
#else
	usleep(i * 1000);	// us
#endif
}


//-----------------------------------------------------------------------------
// setLocalizedDecimalPoint
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters: the C string to process
//
// test with '.' and ',' needed for Linux:
//      Qt put the system decimal point in sprintf() instead of a point!
//      i.e. a comma for a french system!
//
void AU2011_MainWindow::setLocalizedDecimalPoint( char * buff )
{
    char *pch;
    pch = strchr(buff, '.');
    if (pch == NULL)
        pch = strchr(buff, ',');
    if (pch != NULL)
             *pch = LocalizedDecimalPoint;
}
