#ifndef AU2011_MAINWINDOW_H
#define AU2011_MAINWINDOW_H


#include <QtCore/QSettings>
#include <QtCore/QTimer>
#include <QtCore/QDebug>
#include <QtCore/QDir>
#include <QtCore/QFile>

#include <QtGui/QApplication>
#include <QtGui/QMainWindow>
#include <QtGui/QCloseEvent>
#include <QtGui/QInputDialog>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtGui/QProgressDialog>


#ifndef Q_OS_WIN
    #include <termios.h> /* POSIX terminal control definitions */
    #include <fcntl.h>   /* File control definitions */
    #include <errno.h>   /* Error number definitions */
#endif

#include <cmath>



//#define DEBUG_MODE_1                      // ui->adjustAccessCheckBox is checked
//#define DEBUG_MODE                        // allow some debug message send to console

//#define NO_BOOTLOADER                     // *** WARNING *** use this #define only when:
                                            // LCR3 firmware build:
                                            //		without STARTUP.A51 file
                                            //		compliler commande line without 'IV(0x2000)' -> no redirection of Interrup Vector table
                                            //		linker command line without 'CODE(0x2000)' -> code start at 0x0000
                                            // No need for a bootloader loaded in MCU EEPROM -> debug of firmware with the "USB Debug Adapter"

#define FW_VERS_FOR_DAC8811_32      9.0     // for use later! See the commentary to version 2.1.0 below
#define FW_VERS_FOR_CHECKCONNECT    2.3     // minimum firmware version compatible with message "QCON"



// -------------------------------------
#define AU2011_VERSION			"3.0.0"
// -------------------------------------

/* Version history */
                                            // 3.0.0 Works with the new schematic for compensation of input-offsets: new functions
                                            //       to compensate for the input-offsets. (firmware version >= 3.0.0)
                                            //       Continues to work with firmware version >= 2.2.8
                                            //       Menu "Close the port" added.


                                            // 2.1.2 Periodicaly test connection with the hardware. Works with firmware version >= 2.3.0
                                            //       Added a question-message before saving new Rsense value.
                                            //       Added application Icon.

                                            // 2.1.1 Modification of Input_offset adjustement and Phase_offset adjustement functions:
                                            //          Added a warning when the required conditions are not met.

                                            // 2.1.0 With Firmware version >= FW_VERS_FOR_DAC8811_32, works with a double size of DAC8811_Data[],
                                            //              for a better resolution on the DAC8811 gain.

                                            // 2.0.9 Added a warning after the refusal to make an offset adjustment.

                                            // 2.0.8 Modification of "Range" protocol

                                            // 2.0.7 Fixed a bug when bad port is open: "Select port" re-enabled
                                            //       Improved display for very low C or L values

                                            // 2.0.6 Added a QProgressDialog when loading new firmware

                                            // 2.0.5 Improved display for low inductances (nH)

                                            // 2.0.4 If no "locale" translation file found, "locale" set to English.
                                            //       Main window size adjusted to minimum first time the program is loaded.
                                            //       More comments.

                                            // 2.0.3 Checkbox "Max. accuracy" added in the "Preferences" QDockWidget.

                                            // 2.0.2 PT100 added.
                                            //       Set new property "visible" to false for all QDockWidget in the .ui file.
                                            //          -> I used BBEdit (Mac) to edit the AU2011_mainwindow.ui file.

                                            // 2.0.1 English version, Trim bug corrected.

                                            // 2.0.0 Stable version.



#define PI						3.14159265359
#define TWO_PI					6.28318530718
#define TWO_SQRT_2				2.8284271247		// 2 * square root of 2

#define PGA_GAIN_ERROR_10			0.02		// %
#define PGA_GAIN_ERROR_100			0.04		// %
#define PGA_GAIN_ERROR_10_MAX		0.05		// %
#define PGA_GAIN_ERROR_100_MAX		0.2			// %

#define E16_COEFF				1.1548	// 10^1/16
#define GMAX_DAC_16				8.6600	// about 10/E16_COEFF - harware resistance R65 = 470R

// for use later or never...
#define E32_COEFF				1.0746	// 10^1/32
#define GMAX_DAC_32				9.3057	// about 10/E32_COEFF - harware resistance R65 = 510R


#define Z_CHANGE_MODEL			1000.0
#define Q_CHANGE_MODEL			1000.0
#define R_PRIMARY				1
#define L_PRIMARY				2
#define C_PRIMARY				3

#define R_SECONDARY				R_PRIMARY
#define L_SECONDARY				L_PRIMARY
#define C_SECONDARY				C_PRIMARY

#define SERIES					0
#define PARALLEL                1
#define PLAIN					3

#define NO_KIND					0
#define R_KIND					1
#define L_KIND					2
#define C_KIND					3
#define T_KIND					4

#define	 REACTIVE_CIRCUIT		1
#define	 RESISTIVE_CIRCUIT		2

#define WITHOUT_SUFFIX			0
#define WITH_SUFFIX				1

#define NO_ROUNDING				0
#define WITH_ROUNDING			1

#define BUFFER_SIZE				8				// without null terminator
#define MESSAGE_SIZE            19				// without null terminator

#define FREQ_50					0
#define FREQ_60					1

#define NO_TRIM_OPE				0
#define TRIM_SHORT				1
#define TRIM_OPEN				2

#define DISPLAY_R_X				0
#define DISPLAY_U_I				1
#define DISPLAY_ADC				2

#define GO_MEASURE				1
#define	STOP_MEASURE			0

// -------------------------------------------------------------------------------
// *** WARNING don't modify these #define values, there are commands send to the hardware
// -------------------------------------------------------------------------------
#define GO_OFFSET			"GO_OFS\n"		// go for checking offset
#define	GO_LOOP				"GOLOOP\n"		// enter the event loop
#define	RUN					"RUN\n"         // start measurement
#define	HALT				"HALT\n"		// stop measurement
#define	BUF_OFFSET			"B_OFS\n"		// start Buffer offset adjust.
#define	INPUT_OFFSET_I		"I_OFS\n"		// start Input_offset_I adjust.
#define	INPUT_OFFSET_V		"V_OFS\n"		// start Input_offset_V adjust.
#define	SINUS_OFFSET		"S_OFS\n"		// start Sinus offset adjust.
#define	FREQ_1				"FREQ1\n"		// set Freq_range = 0
#define	FREQ_2				"FREQ2\n"		// set Freq_range = 1
#define	FREQ_3				"FREQ3\n"		// set Freq_range = 2
#define OFFSET_OK			"OFS_OK\n"		// end offset adjust.
#define	S_TRIM_SHORT		"TRIM_S\n"		// set trim (short)
#define	S_TRIM_OPEN			"TRIM_O\n"		// set trim (open)
#define AVERAG_x			"AVG_%d\n"		// x = 1 to 9
#define	QUERY_AVERAGE		"AVG_V\n"		// average value?
#define	R_100K				"R100K "		// save Rsense value (100k)
#define	R_10K				"R10K "			// save Rsense value (10k)
#define	R_1K				"R1K "			// save Rsense value (1k)
#define	R_100R				"R100R "		// save Rsense value (100R)
#define	QUERY_R_100K		"QR100K\n"		// Rsense (100k) value?
#define	QUERY_R_10K			"QR10K\n"		// Rsense (10k) value?
#define	QUERY_R_1K			"QR1K\n"		// Rsense (1k) value?
#define	QUERY_R_100R		"QR100R\n"		// Rsense (100R) value?
#define	SAVE_FREQ			"S_FREQ\n"		// save frequency
#define	GO_PHASE_ADJ		"GO_PHI\n"		// start Phi adjust.
//#define	NEW_PHASE		"PHIDAT "		// Phi data, used with addition of a parameter.
#define	SAVE_PHASE			"SAVPHI\n"		// save Phi data
#define	GO_PGA_ADJ			"GO_PGA\n"		// start PGA adjust.
//#define	NEW_PGA			"PGADAT "		// PGA data, used with addition of a parameter.
#define	SAVE_PGA			"SAVPGA\n"		// save PGA data
#define	EXIT_PHI_PGA		"EXPHPG\n"		// exit Phi/PGA adjust.
#define	FIRMW_UPDATE		"FWUPDA\n"		// start Firmware Update
#define START_BOOTLOADER	"START\n"		// bootloader Start
#define LINE5				"LINE5\n"		// line 50Hz
#define LINE6				"LINE6\n"		// line 60Hz
#define	UPDATE				"UPDATE\n"		// update firmware
#define	UPDATE2				"UPDATE2\n"		// erase bank2 & update firmware
#define QUERY_CONNECTED     "QCON\n"        // Tests if the device is still connected
//#define MAX_INDEX_V		"INDXV "		// used with addition of a parameter.
//#define MAX_INDEX_I		"INDXI "		// used with addition of a parameter.

// -------------------------------------------------------------------------------
// *** WARNING don't modify these #define values, there are strings send by the hardware
// -------------------------------------------------------------------------------
#define GO_APP				"GO_APP"
#define VERSION				"Version"
#define SET_BUFFER_OFS		"SetBufferOffset"
//#define SET_INPUT_OFS		"SetInputOffset"
#define SET_INPUT_OFS_V		"SetInputOffset_V"
#define SET_INPUT_OFS_I		"SetInputOffset_I"
#define SET_SINUS_OFS		"SetSinusOffset"
#define	TRIM_OK				"TRIM_ok"
#define	BAD_TRIM			"Bad_TRIM"
#define INIT				"INIT"
#define UPDATE_READY		"UPDATE_FIRMWARE_READY"
#define MEMORY_FAIL			"MEMORY_FAIL"
#define NEED_UPDATE			"NEED_UPDATE"
#define READY_RECEIVE		"READY_RECEIVE"
#define OVERFLOW_MSG		"OVERFLOW"
#define OK_CONNECTED        "CONNECTED"
#define IMPROVED_OFFSET     "IMPROVED_OFFSET"
#define BAD_OFFSET          "BAD_OFFSET"

namespace Ui {
	class AU2011_MainWindow;
}

class AbstractSerial;

class AU2011_MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit AU2011_MainWindow(QWidget *parent = 0);
    ~AU2011_MainWindow();

private:
	Ui::AU2011_MainWindow *ui;

    AbstractSerial *port;
    QStatusBar *barreEtat;
    QSettings *settings;

    QAction *exitAction;
    QAction *setPortNameAction;
    QAction *closeThePortAction;
    QAction *aboutAction;
    QAction *aboutQtAction;
    QAction *prefAction;
    QAction *resistorAction;
    QAction *bufferOffsetAction;
    QAction *inputOffsetActionV;
    QAction *inputOffsetActionI;
    QAction *sinusOffsetAction;
    QAction *inputPhaseAction;
    QAction *PGAPhaseAction;
    QAction *FirmwareUpdateAction;
    QAction *ParamSortAction;
    QAction *SortAction;
    QAction *OkForSortAction;

    QMenu *portMenu;
    QMenu *outilsMenu;
    QMenu *reglagesMenu;
    QMenu *trierMenu;
    QMenu *helpMenu;

    QTimer *trimTimer;                  // used for blinking the trim warning
    QTimer *connectionTimer;            // used for the test of the connection with the hardware

    void closeEvent(QCloseEvent *event);
    void setDelay(unsigned int);
	void createActions(void);
	void createMenus(void);
    void addMenus(void);
    bool ParseInputString (void);
    void ReadInitData(void);
    void ReadRequest(void);
    void WarningBeforeClosing(void);
    bool GetFileNameHex(void);
    void SendFichierHex(void);
    void ClearDataInWindow();
    void Display_Offset(void);
    void Display_PhiData(void);
    void ProcessAllParameters (void);
    void ProcessCircuitXs (void);
    void ProcessCircuitRs (void);
    void Display_Primary_Value (void);
    void Display_Secondary_Value (void);
    void Display_Frequency (void);
    void Display_Q_D (void);
    void Display_PHI(void);
    void Display_Z(void);
    void Display_VxRs(void);
    void Display_IxXs(void);
    void Display_Ranges(void);
	void Display_Accuracy(void);
    void Display_Symboles(void);
    double Gain_Error(void);
    void Rounding_double(double * value, unsigned int coeff);
	char Convert_Value_to_String (char accuracy, bool suffix,
								  char Comp_Kind, bool adjust_format);
    double PT100_Temperature(double RT);
    void setLocalizedDecimalPoint( char * buff );

	int	 timeOut;
    bool ok;
    char Buffer[BUFFER_SIZE+1];

	QLocale theLocal;
    char LocalizedDecimalPoint;

    QString ExpoBuf;

    bool allow_DAC8811_32;
    bool improvedOffset;
    bool messageReceived;
    bool checkConnectionAllowed;
    bool initialization;
	bool viewTrimWarning;
    bool onMeasurement;
    bool onOffsetAdjust;
    bool onResistorsAdjust;
    bool onTrimAdjust;
    bool onInputPhaseAdjust;
    bool onPGAPhaseAdjust;
    bool onFirmwareUpdate;
    bool isCapacitive;
	bool MaxPrecision;
	char SP_mode;					// 0 for series, 1 for parallel
    char AutoModel;
    char AutoParameter;
    char lineCode;
    char inputString[127];
    char primaryKind;
    char circuitKind;
    char secondaryKind;
    char savedChoice;
    char trimOperation;
    QString inputQString;
    QString stringValue;
    QString portDeviceName;
    QString nomFichierHex;
	QString chaineSort;
    QString firmwareVersion;
    QString bootloaderVersion;

    double RsValue;
    double XsValue;
    uint  FreqValue;
    uint  Ranges, oldRanges;
    uchar Range_value;
    uchar dac_U;
    uchar dac_I;
    char  Limite_DAC_Index_V;
    char  Limite_DAC_Index_I;
    double VppValue;
    double IppValue;
    double ADC_Vpp;
    double ADC_Ipp;
	char  initFreqCode;				// 1 or 2 or 3
    bool  trimOpenState;
    bool  trimShortState;
    char  averagingValue;
	char  rsenseButtonSelected;		// 1 for 100R, 2 for 1K , 3 for 10K , 4 for 100K
    double initialRsenseValue;

    QChar  Omega;
    double CsValue;
    double LsValue;
    double CpValue;
    double LpValue;
    double ZValue;
    double PhiValue;
    double QValue;
    double SP_coeff;
    double PrimaryValue;
    double Value_to_Print;
    double PhiData;
    double PhiMultiplier;
    double SortToleranceValue;
    double SortValue;
	char  SortKind;
    double SortSuffix;
    double Exx_Coeff;
    double GMax_DAC;

private slots:
	void aboutAct(void);
    void aboutQtAct(void);
    void preferenceAct(void);
	void GetPortName(void);
    void SendConnectedRequest(void);
	void PT100_Checked(int);
	void SetMenuReglages(int);
	void SetUIDisplay(int);
	void SetResistors(void);
	void SetBufferOffset(void);
    void SetInputOffset_V(void);
    void SetInputOffset_I(void);
	void SetSinusOffset(void);
	void SetInputPhase(void);
	void closeOffsetWidget(void);
	void SetPGAPhase(void);
    void setIndex_V_Value(int Value);
    void setIndex_I_Value(int Value);
	void setPhaseAdjustMultiplier(void);
	void setPlusMinusPhaseAdjustMultiplier(int);
	void validatePhiWidget(void);
	void closePhiWidget(void);
	void increasePhiDataValue(void);
	void decreasePhiDataValue(void);
	void setFirmwareUpdate(void);
	void validateResistorWidget(void);
	void closeResistorWidget(void);
	void changeSelectedResistor(void);
	void setSortingParameters(void);
	void validateParamSortingWidget(void);
	void closeParamSortingWidget(void);
	void loadComponantParameters(void);
	void SortValueChanged(double);
	void sortAllowed(void);
	void processSort(void);
	void hideSortResultLabel(void);
	void OpenSerialPort(void);
	void CheckCom(void);
	void updateTrimView(void);
	void CloseApplication(void);
	void RunStopMeasure(void);
	void switchLedOff(void);
	void Trim(void);
	void slotRead(void);
	void setFrequency(bool checked);
	void setAutoMode(bool checked);
	void setLineFrequency(bool checked);
	void setAverageValue(int);
    void setMaxPrecision(int);
    void reactivateMenus(void);
};

#endif // AU2011_MAINWINDOW_H
