// --------------------------------------------------------------------------------------------------
// This program is the GUI part of AU2011 LCR meter.
// USB is used for the communincation between the AU2011 board and the Computer.
// The USB UART interface Integrated Circuit is a FT232R from FTDI Chips.
//
// A Virtual Com Port driver (VCP  driver) must be installed on MacOSX and Windows (and Linux if kernel version < 2.6.31)
//
// Sources files are with texts in english language, with comments for translator in French.
// Put your translation file (AU2011_xx.qm) for your "locale" in the AU2011 application directory.
// Put also the localized qt_xx.qm file (required for translation of dialogs and buttons) in same directory.
//
// Without translation file for your "locale", application is in english (texts and numbers)
//
// You can also force a language by editing the "settings" file and write in the [General] field:
//      MyLocale="fr_FR" for French/France
//   or
//      MyLocale="de_CH" for German/Switzerland
//   etc...
//
// Obviously it is necessary that the corresponding translation files exist in the application directory!
//
// To force to stay in English, without taking account of translation files, write MyLocale="C"
//
// Settings file is located
// - OSX and Linux: $HOME/.config/FRAUBRJ/AU2011 x.x.x.ini
// - Windows: %APPDATA%\FRAUBRJ\AU2011 x.x.x.ini
//
// If the version of the AU2011 program changes, you will need to edit this file again.
//
// --------------------------------------------------------------------------------------------------


#include <QtCore/QTranslator>

#include "AU2011_mainwindow.h"



int main(int argc, char *argv[])
{
	QApplication a(argc, argv);

    QSettings * mySettings;
    QTranslator translator1, translator2;
    QString locale, localeAppName, localeQtName;
    QDir dir;
    int slashPos;

    // look in setting file for specific locale
    mySettings = new QSettings( QSettings::IniFormat, QSettings::UserScope,
                                                "FRAUBRJ", "AU2011 " + QString( AU2011_VERSION ) );
    if (mySettings->contains("myLocale"))
        locale = mySettings->value("myLocale").toString( );
    else
        locale = QLocale::system().name();

    delete mySettings;
    qDebug() << "locale = " << locale;

    if (locale == "C" || locale.contains("en_US"))
    {
       QLocale::setDefault(QLocale::C);    // set "locale" to English/UnitedStates
       goto runApplication;
    }

    localeQtName = "qt_" + locale + ".qm";

    // construct the fileName for "locale" translation file
    localeAppName = QCoreApplication::arguments().at(0);	// with full path
    slashPos = localeAppName.lastIndexOf(QDir::separator ());
    localeAppName.remove(0, slashPos + 1);	// remove path

#if defined (Q_OS_MACX)
    // go up to application directory, where the translation files are located
    dir = QDir::currentPath();		// "MacOS" directory, the directory of the Unix file
    dir.cdUp();						// "Contents" directory
    dir.cdUp();						// "AU2011.app" directory
#elif defined(Q_OS_UNIX)
    // to do for Linux, if needed
#elif defined (Q_OS_WIN32)
    // remove ".exe"
    slashPos = localeAppName.lastIndexOf(".");
    localeAppName.remove(slashPos, 100);
#endif

    // at this point "localeAppName" is the name of the application ("AU2011" if it was not renamed)
    localeAppName.append("_" + locale + ".qm");

    if (translator1.load(localeAppName))
    {
        //qDebug() << "ok, locale file for translation";
        a.installTranslator(&translator1);
        if (translator2.load(localeQtName))
            a.installTranslator(&translator2);
    }
    else
    {
        //qDebug() << "no locale file for translation";
        QLocale::setDefault(QLocale::C);    // set "locale" to English/UnitedStates
    }

runApplication:

    AU2011_MainWindow w;
    w.show();

    return a.exec();
}
